//---------------------------------------------------------
// Header file for a simple logic analyzer
//
// This logic analyzer is implemented in an FPGA that is
// connected up to an Atmel ATmega2561 AVR microcontroller
//
// It is intended to be used as an aid in debugger FPGA
// hardware.  It is 32 bits wide and has a very simple trigger
// mode.
//-------------------------------------------------------------

#ifndef LOGICANALYZER__H
#define LOGICANALYZER__H

#define LA_BASE		(0xf080)
#define LA_READADR	*((volatile unsigned int *)(LA_BASE))
#define LA_WRITEADR	*((volatile unsigned int *)(LA_BASE+2))
#define LA_RAM		*((volatile unsigned char *)(LA_BASE+4))
#define LA_TRIG0	*((volatile unsigned char *)(LA_BASE+6))
#define LA_TRIG1	*((volatile unsigned char *)(LA_BASE+7))
#define LA_TRIG2	*((volatile unsigned char *)(LA_BASE+8))
#define LA_TRIG3	*((volatile unsigned char *)(LA_BASE+9))
#define LA_TRIG4	*((volatile unsigned char *)(LA_BASE+10))
#define LA_TRIG5	*((volatile unsigned char *)(LA_BASE+11))
#define LA_TRIG6	*((volatile unsigned char *)(LA_BASE+12))
#define LA_TRIG7	*((volatile unsigned char *)(LA_BASE+13))
#define LA_CSR		*((volatile unsigned char *)(LA_BASE+15))

#define LA_READADR_P	((volatile unsigned int *)(LA_BASE))
#define LA_WRITEADR_P	((volatile unsigned int *)(LA_BASE+2))
#define LA_RAM_P		((volatile unsigned char *)(LA_BASE+4))
#define LA_TRIG0_P		((volatile unsigned char *)(LA_BASE+6))
#define LA_TRIG1_P		((volatile unsigned char *)(LA_BASE+7))
#define LA_TRIG2_P		((volatile unsigned char *)(LA_BASE+8))
#define LA_TRIG3_P		((volatile unsigned char *)(LA_BASE+9))
#define LA_TRIG4_P		((volatile unsigned char *)(LA_BASE+10))
#define LA_TRIG5_P		((volatile unsigned char *)(LA_BASE+11))
#define LA_TRIG6_P		((volatile unsigned char *)(LA_BASE+12))
#define LA_TRIG7_P		((volatile unsigned char *)(LA_BASE+13))
#define LA_CSR_P		((volatile unsigned char *)(LA_BASE+15))


#define LA_QUALIFIER	*((volatile unsigned int *)(0xf004))
#define LA_QUALIFIER_P	((volatile unsigned int *)(0xf004))

/*!
 * Logic analyzer structures
 */

typedef struct _la_status LA_STATUS;

struct _la_status {
	unsigned short WordsWritten;
	unsigned short BytesRead;
	char State;
};

/*!
 * Control Register Defines
 */

#define LA_CR_CLEAR_READ_ADDRESS		(0)
#define LA_CR_CLEAR_WRITE_ADDRESS		(1)
#define LA_CR_ARM						(2)

/*!
 * Status register defines
 */

#define	LA_STATE_IDLE		0
#define LA_STATE_ARMED		1
#define LA_STATE_TRIGGERED	2
#define LA_STATE_DONE		3

//----------------------------------
// Function prototypes
//----------------------------------

extern void LA_SetTrigger(char *tw);
extern void LA_Arm(void);
extern char *LA_Read(int n,unsigned short *pN);
extern void LA_Reset(void);
extern void LA_GetStatus(LA_STATUS *pS);

#endif

