#include <stdio.h>
#include <stdlib.h>
#include "io.h"
#include "commProto.h"
#include <string.h>
#include "fcntl.h"
#include "LogicAnalyzer.h"
#include "Synth.h"

TCB TCom,TCmd;
int CommandPortHandle;
MESSAGE_QUEUE *MainMSG;

//----------------------------------------------------------------------
// Functions for sending things back to host computer
//----------------------------------------------------------------------

void MMSendAck(int Handle,unsigned short AckCMD,char Status,char *data,int l)
{
	////////////////////////////////////////////////////////////
	//send ack back to host
	//
	// parameter:
	//	AckCMD........command that is being acked
	//	Status........status of operation
	//	data..........pointer to data returned by operation
	//	l.............number of data bytes to return
	///////////////////////////////////////////////////////////
	int i = 0;
	int j,n;
	char *s = malloc(128);
	n = l + 5;
	s[i++] = LO(n);
	s[i++] = HI(n);
	s[i++] = MM_OUT_ACK;	//lsb of command
	s[i++] = 0;			//msb of command
	s[i++] = LO(AckCMD);	//lsb of command being ACKed
	s[i++] = HI(AckCMD);	//msb of command being ACKed
	s[i++] = Status;
	if(l)
	{
		for(j=0;j<l;++j,++i)	//load payload into buffer
			s[i] = data[j];
	}
	_write(Handle,s,i);		//send command
	free(s);
}


void ExecuteCommandTask(void *a)  __attribute__ ( ( noreturn ) );
void ExecuteCommandTask(void *a)
{
	int sr;
	MSG *msg;
	char *s;
	int count;
	WINDOW *win;
//	float *pF;
	unsigned short *pU;
	unsigned long *pL;
	unsigned char nuC;
//	unsigned long nL;
	unsigned short nUS;

	win = newwin(16,30,16,30,NULL,FG(BLACK) | BG(RED),WIN_BOXED);
	WinSetTitle(win,"Commands");
	scrollok(win,1);
	InitMessageQueueManager();
	sr = Disable();
	MainMSG = MqInit(32,"Main Msg");
	count = 0;
	Enable(sr);
	while(1)
	{
		msg = MqGet(MainMSG);
		wprintw(win,"Got Message %d\n",msg->MsgCmd);
		switch(msg->MsgCmd)	//process Message
		{
			case MM_TEST:
//				msg->Payload[20] = 0;
//				l = strlen(msg->Payload);
				wprintw(win,"%s\n",msg->Payload);
				s = malloc(128);
				strcpy(s,"MM_TEST ");
				itoa(count++,&s[8],10);
				MMSendAck(msg->Handle,MM_TEST,0,s,strlen(s) + 1);
				free(s);
				break;
			case MM_SYNTH_EN:
				nuC = msg->Payload[0];	//get enable bit
				EnableSynth(nuC);
				MMSendAck(msg->Handle,MM_SYNTH_EN,0,NULL,0);
				break;
			case MM_ARM_LA:
				LA_Arm();
				MMSendAck(msg->Handle,MM_ARM_LA,0,NULL,0);
				break;
			case MM_READ_LA:
				pU = (unsigned short *)&msg->Payload[0];
				s= LA_Read(*pU,&nUS);
				wprintw(win,"%04x\n",(int)s);
				MMSendAck(msg->Handle,MM_READ_LA,0,s,nUS);
				free(s);
				break;
			case MM_SET_LA_TRIG:
				LA_SetTrigger(msg->Payload);
				pL = (unsigned long *)&msg->Payload;
				{
					long tw,tl;
					tw = *pL++;
					tl = *pL;
					wprintw(win,"%lx %lx\n",tw,tl);
				}
				MMSendAck(msg->Handle,MM_SET_LA_TRIG,0,NULL,0);
				break;
			case MM_GET_LA_TRIG:
				break;
			case MM_SET_OSC_FREQ:
				pU = (unsigned short *)msg->Payload;
				pL = (unsigned long *)&msg->Payload[2];
				SetOscFreq(*pU, *pL);
				MMSendAck(msg->Handle,MM_SET_OSC_FREQ,0,NULL,0);
				break;
			case MM_GET_OSC_FREQ:
				{
					ACKOSC *pAK;
					pAK = (ACKOSC *)malloc(sizeof(ACKOSC));
					pAK->Ch = *(unsigned short *)msg->Payload;
					pAK->Freq = GetOscFreq(pAK->Ch);
					wprintw(win,"%d  %lx\n",pAK->Ch,pAK->Freq);
					MMSendAck(msg->Handle,MM_GET_OSC_FREQ,0,(char *)pAK,sizeof(ACKOSC));
					free(pAK);
				}
				break;
			case MM_SET_EV:
				{
					EVPARAMS *pEVp;
					unsigned short nEv,nUn,*pU;
					pU = (unsigned short *)msg->Payload;
					nEv = *pU++;	//get evelope generator bank
					nUn = *pU++;	//get evelope generator unit number
					nUn *= 2;
					pEVp = (EVPARAMS *)pU;
					pU = (unsigned short *)nEv?SYNTH_EV2_AD_PARAMS_P:SYNTH_EV1_AD_PARAMS_P;
					pU[nUn] = pEVp->Decay;
					pU[nUn+1] = pEVp->Attack;
					pU = (unsigned short *)nEv?SYNTH_EV2_SR_PARAMS_P:SYNTH_EV1_SR_PARAMS_P;
					pU[nUn] = pEVp->Sustain;
					pU[nUn+1] = pEVp->Release;
				}
				break;
			case MM_GET_EV:
				{
					unsigned *pUD = malloc(sizeof(unsigned) * 2 + sizeof(EVPARAMS));
					EVPARAMS *pEVp = (EVPARAMS *)&pUD[2];
					unsigned *pU;
					pUD[0] = *(unsigned *)msg->Payload;
					pUD[1] = *(unsigned *)&msg->Payload[2];
					pU = (unsigned *)pUD[0]?SYNTH_EV1_AD_PARAMS_P:SYNTH_EV2_AD_PARAMS_P;
					pEVp->Attack = pU[pUD[1]];
					pEVp->Decay = pU[pUD[1]+1];
					pU = (unsigned *)pUD[0]?SYNTH_EV1_SR_PARAMS_P:SYNTH_EV2_SR_PARAMS_P;
					pEVp->Sustain = pU[pUD[1]];
					pEVp->Release = pU[pUD[1]+1];
					MMSendAck(msg->Handle,MM_GET_EV,0,(char *)pUD,
									sizeof(unsigned) * 2 + sizeof(EVPARAMS));
					free(pUD);
				}
				break;
			case MM_SET_FLT_FREQ:
				break;
			case MM_GET_FLT_FREQ:
				break;
			case MM_SET_FLT_Q:
				break;
			case MM_GET_FLT_Q:
				break;
			case MM_RESET_LA:
				LA_Reset();
				MMSendAck(msg->Handle,MM_RESET_LA,0,(char *)0,0);
				break;
			case MM_STATUS_LA:
				{
					LA_STATUS *pSTS = malloc(sizeof(LA_STATUS));
					LA_GetStatus(pSTS);
					MMSendAck(msg->Handle,MM_STATUS_LA,0,(char *)pSTS,sizeof(LA_STATUS));
					free(pSTS);
				}
				break;
			case MM_SETQUAL_LA:
				pU = (unsigned short *)msg->Payload;
				LA_QUALIFIER = *pU;
				MMSendAck(msg->Handle,MM_SETQUAL_LA,0,(char *)0,0);
				break;
			case MM_GETQUAL_LA:
				MMSendAck(msg->Handle,MM_GETQUAL_LA,0,(char *)LA_QUALIFIER_P,2);
				break;
			case MM_PEEK:
				nUS = *((volatile unsigned short *)msg->Payload);
				nuC = *((volatile unsigned char *)nUS);
				MMSendAck(msg->Handle,MM_PEEK,0,(char *)&nuC,1);
				break;
			case MM_SINGLESTEP:
				SYNTH_CMDREG = BIT(SYNTH_SINGLE_STEP);
				MMSendAck(msg->Handle,MM_SINGLESTEP,0,NULL,0);
				break;
			case MM_GETDEBUG:
				MMSendAck(msg->Handle,MM_GETDEBUG,0,(char *)SYNTH_DEBUG_DATA_P,sizeof(SYNTH_DEBUG));
				break;
			case MM_EV_TRIGGER:
				{
					unsigned nEv,nUn,*pU;
					unsigned char *pUC;

					pU = (unsigned *)msg->Payload;
					nEv = *pU++;	//get evelope generator bank
					nUn = *pU++;	//get evelope generator unit number
					pUC = (unsigned char *)nEv?SYNTH_EV2_CSR_P:SYNTH_EV1_CSR_P;
					pUC[nUn] = (unsigned char)*pU;
				}
				break;
			case MM_EV_STATUS:
				break;
			case MM_SET_SSCHANNEL:
				{
					unsigned short *pUS;
					pUS = (unsigned short *)msg->Payload;
					*((unsigned short *)0xf022) = *pUS;
					MMSendAck(msg->Handle,MM_SET_SSCHANNEL,0,(char *)pUS,sizeof(unsigned short));
				}
				break;
			case MM_GET_SSCHANNEL:
				MMSendAck(msg->Handle,MM_GET_SSCHANNEL,0,(char *)0xf022,sizeof(unsigned short));
				break;
			default:
				wprintw(win,"BAD COMMAND\n");
				break;
		}
		DeleteMSG(msg);
	}
}

void CommTask(void *arg)  __attribute__ ( ( noreturn ) );
void CommTask(void *arg)
{
	WINDOW *win;
	int c;
	union {
		short count;
		char cV[2];
	}C;
	int State;
	int index=0;
	union {
		char cV[2];
		short sV;
	}Cmd;
	MSG *msg=NULL;

	CommandPortHandle = _open("COM1",FCNTL_WRONLY | FCNTL_RDONLY);
	win = newwin(10,20,16,60,NULL,FG(BLACK) | BG(GREEN),WIN_BOXED);
	WinSetTitle(win,"Comms");
	scrollok(win,1);

	State = MON_STATE_IDLE;
	while(1)
	{
		c =  _get(CommandPortHandle);
		switch(State)
		{
			case MON_STATE_IDLE:
				C.cV[0] = (char)c;
				index = 0;
				State = MON_STATE_COLLECT_SIZE;
				break;	
			case MON_STATE_COLLECT_SIZE:
				C.cV[1] = (char)c;
//				wprintw(win,"Size = %d\n",C.count);
				State = MON_STATE_COLLECTCMD;
				break;
			case MON_STATE_COLLECTCMD:
				Cmd.cV[index++] = (char)c;
				if(index == 2)
				{
					msg = NewMSG(C.count+2);	//new message object
					msg->MsgCmd = Cmd.sV;
					index = 0;
					State = MON_STATE_COLLECTDATA;
				}
				break;
			case MON_STATE_COLLECTDATA:
				msg->Payload[index++] = (char)c;
//				wprintw(win,"Index = %d\n",index);
				if(index == (C.count-2))
				{
					msg->PayloadSize = index;
					msg->Handle = CommandPortHandle;
					MqPut(MainMSG,msg);
					State = MON_STATE_IDLE;
					wprintw(win,"C.c=%d\n",C.count);
				}
				break;
		}
	}
}

void InitCommProtocol(void)
{
	CreateTask(&TCmd,ExecuteCommandTask,384,5,"Cmnds",NULL);
	Insert(&ActiveTasks,&TCmd);
	CreateTask(&TCom,CommTask,384,4,"Comms",NULL);
	Insert(&ActiveTasks,&TCom);
}
