#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "system.h"
#include "task.h"
#include "rs232.h"
#include "xserial.h"
#include "MessageQueue.h"
#include "blockalloc.h"
#include "display.h"
#include "HeapManager.h"
#include "CURSES.h"
#include "video.h"
#include "cio.h"
#include "io.h"
#include "time.h"
#include "commProto.h"
#include "TimeEvents.h"
#include "SimpleDDS.h"
#include "TimerServices.h"
#include "spi.h"
#include "mmc.h"

#define STACKSIZET	256
#define STACKSIZE_EC	256


////////////////////////////////////////////////////////////////////////////
// external interrupt 4 defines
////////////////////////////////////////////////////////////////////////////

#define TIMER_CNTRL       *((volatile unsigned char *)0xf040)   //address of counter low
#define TIMER_CNTRH       *((volatile unsigned char *)0xf041)
#define TIMER_RLDLO       *((volatile unsigned char *)0xf042)
#define TIMER_RLDHI       *((volatile unsigned char *)0xf043)
#define TIMER_LCHLO       *((volatile unsigned char *)0xf044)
#define TIMER_LCHHI       *((volatile unsigned char *)0xf045)
#define TIMER_STATUS      *((volatile unsigned char *)0xf046)   //read only address
#define TIMER_DOLATCH     *((volatile unsigned char *)0xf046)   //write only address
#define TIMER_CTRL        *((volatile unsigned char *)0xf047)

TCB T2,TIdle,TM,TSch,TEvnt;
ECB TimerSem;
ECB *OneSecond;
MESSAGE_QUEUE *EventMSG;

volatile long Idle,TaskTwoCount;


volatile int Ticks;


void FillSectorBuffer(long Sector,char *b)
{
	long *lp;
	int i;

	lp = (long *)b;

	for(i=0;i<128;++i)
		lp[i] = Sector;
}

char MakeHex(char v)
{
	if(v > 9)
		v += 'A' - 10;
	else
		v += '0';
	return v;
}

int CharToHex(char v,char *s,char c)
{
	int i = 0;

	s[i++] = MakeHex((v >> 4) & 0x0f);
	s[i++] = MakeHex(v & 0x0f);
	if(c)
		s[i++] = c;
	return i;
}

int UnsignedToHex(unsigned v,char *s,char c)
{
	char *t = (char *)&v;
	int i = 0;

	++t;
	i += CharToHex(*t--,&s[i],0);
	i += CharToHex(*t,&s[i],c);
	return i;
}

void HexDump(int fd,char *b,int n)
{
	char *s = NewBlock();
	int i,j,l;
	unsigned adr;

	adr = (unsigned)b;

	for(j=0;j<n;j+=8)
	{
		l = 0;
		l += UnsignedToHex(adr,&s[l],' ');
		for(i=0;i<8;++i)
		{
			l += CharToHex(b[i+j],&s[l],(i==7)?' ':'-');
		}
		for(i=0;i<8;++i)
		{
			if(isprint(b[i+j]))
				s[l++] = b[i+j];
			else
				s[l++] = '.';
		}
		s[l++] = '\r';
		s[l++] = '\n';
		_write(fd,s,l);
		adr += 8;
	}
	DeleteBlock(s);
}

void wHexDump(WINDOW *w,char *b,int n)
{
	char *s = malloc(256);
	int i,j,l;
	unsigned adr;

	adr = (unsigned)b;

	for(j=0;j<n;j+=8)
	{
		l = 0;
		l += UnsignedToHex(adr,&s[l],' ');
		for(i=0;i<8;++i)
		{
			l += CharToHex(b[i+j],&s[l],(i==7)?' ':'-');
		}
		for(i=0;i<8;++i)
		{
			if(isprint(b[i+j]))
				s[l++] = b[i+j];
			else
				s[l++] = '.';
		}
		s[l++] = 0;
		wprintw(w,"%s\n",s);
		adr += 8;
	}
	free(s);
}

////////////////////////////////////////////////////////////////////////////////

void ExtInt4Init(void)
{
	EIMSK |= BIT(INT4);
	TIMER_DOLATCH = 0;
	EIFR = BIT(INT4);
	TIMER_CTRL = 0xc4;	//enable counter and counter interrupt
}

volatile int I4Ticks;

void INT4_vect(void) __attribute__ ((naked));
void INT4_vect(void)
{
	SAVE_IRQ();
	++InterruptCount;
	TIMER_DOLATCH = 0;
	EIFR = BIT(INT4);
	++I4Ticks;
	PostSemaphoreIrq(&TimerSem,0);
	ExitInterrupt();
	RESTORE_IRQ();
}

void Timer0Init(void)
{
	TCCR0B = 0x00;	//stop
	ASSR  = 0x00;	//async mode
	TCNT0 = 0x80;
	OCR0A = 0xff;
	TCCR0B = BIT(CS01) | BIT(CS00); //start timer
	TIMSK0 = BIT(OCIE0A);
}

void TIMER0_COMPA_vect(void) __attribute__ ((naked));
void TIMER0_COMPA_vect(void)
{
	SAVE_IRQ();
	TCNT0 = 0x80;
	++InterruptCount;
	++Ticks;
	TimerTicker();
	ExitInterrupt();
	RESTORE_IRQ();
}

void reset(void)
{
	*((volatile char *)0xf001) = 0x5a;
}

void PortInit(void)
{
	PORTA = 0x00;	//IDE Data LSB
	DDRA  = 0x00;	//leave as inputs unless writing
	PORTB = 0xff;	//IDE control lines;
	DDRB  = 0xff;	//output port
	PORTC = 0x00;	//IDE Data MSB
	DDRC  = 0x00;	//leave as inputs unless writing
	PORTD = 0x00;
	DDRD  = 0x00;
	PORTE = 0x00;
	DDRE  = 0x00;
	PORTF = 0x00;
	DDRF  = 0x00;	//these are all outputs
	PORTG = 0;		//Control Port
	DDRG  = BIT(3);	//Bit 3 is output (reset line)
	PORTG |= BIT(3);	//reset
	PORTG &= ~BIT(3);
}




extern char __end_of_heap;

void TaskTwo(void *arg)  __attribute__ ( ( noreturn ) );
void TaskTwo(void* arg)
{
	WINDOW *win2;
	unsigned char v;
	char f;
	int c;

	v = 1;
	f = 1;
	c = 0;
	int count = 0;
	int t = 20;

	win2 = newwin(16,30,32,30,NULL,FG(BLUE) | BG(YELLOW),WIN_BOXED);
	scrollok(win2,1);
	WinSetTitle(win2, "Task Two");
	while(1)
	{
		PendSemaphore(&TimerSem,0);
		--t;
		if(t == 0)
		{
			t = 20;
			PostSemaphore(OneSecond,0);
		}
		if(f)
		{
			v <<= 1;
			if(v & 0x80)
				f = 0;
		}
		else
		{
			v >>= 1;
			if(v & 0x01)
				f = 1;
		}
		SYSTEM_LED = v;
		wprintw(win2,"This is COUNT = %d\n",count++);
	}
}

void MemTestTask(void *arg)  __attribute__ ( ( noreturn ) );
void MemTestTask(void * arg)
{
	WINDOW *win;
	extern unsigned int FreeMallocSpace(void);
	unsigned char *m;
	unsigned char c,seed;
	unsigned int mSize;
	unsigned int i;
	char flag;
	long errors;
	char *s;
	int fd;


	win = newwin(16,30,32,0,NULL,FG(BLUE) | BG(VIOLET),WIN_BOXED);
	WinSetTitle(win,"Mem Test");
	s = malloc(128);
	mSize = FreeMallocSpace();
	mSize -= 8000;
	wprintw(win,"Memory Tested Size = %u",mSize);
	m = malloc(mSize);
	errors = 0;
	fd = _open("XR",CIO_WRITE_ONLY | CIO_READ_ONLY);
	while(1)
	{
		seed = 0x01;
		while(seed < 255)
		{
			c = seed;
			flag = 0;
			for(i=0;i<mSize;++i)
			{
				m[i] = c;
				if(flag)
				{
					c >>= 1;
					if(c & 1) flag = 0;
				}
				else
				{
					c <<= 1;
					if(c & 0x80) flag = 1;
				}
				
			}
			c = seed;
			flag = 0;
			for(i=0;i<mSize;++i)
			{
				if(m[i] != c) errors++;
				if(flag)
				{
					c >>= 1;
					if(c & 1) flag = 0;
				}
				else
				{
					c <<= 1;
					if(c & 0x80) flag = 1;
				}
			}
			i = sprintf(s,"Pat = %02x  Errors = %ld\r\n",seed & 0x0ff,errors);
			_write(fd,s,i);
			wmove(win,1,0);
			wprintw(win,"Pat = %02x  Errors = %ld",seed & 0x0ff,errors);
			seed = (seed << 1) | 1;
		}
	}
}

void ScheduleTask(void *arg)  __attribute__ ( ( noreturn ) );
void ScheduleTask(void *arg)
{
	WINDOW *win;
	int c=0;
	extern char script_txt[];
	extern int runScript(char *Script,WINDOW *wOut);

	win = newwin(12,80,48,0,NULL,FG(BLACK) | BG(CYAN),WIN_BOXED);
	WinSetTitle(win,"Schedule");
	scrollok(win,1);
	InitEventTriggers(16);
	while(1)
	{
		PendSemaphore(OneSecond,0);
		wprintw(win,"%s\n",(c & 1)?"Tick":"Tock");
		++c;
		runScript(script_txt,win);
	}
}

void DisplayTask(int line,WINDOW *w,TCB *t)
{
	wmove(w,line,0);
	wprintw(w,"%04x:%s:SB: %04x SP:%04x Free:%3u  Min:%3u Swaps:%5u",
		(unsigned)t,t->name,(unsigned)t->stacktop,(unsigned)t->stack,(unsigned)(t->stack-t->stacktop),
		MeasureStackUsage(t->stacktop,t->stacksize),t->TcbSwaps);
}

void DisplaySem(int line,WINDOW *w, ECB *e)
{
	wmove(w,line,0);
	wprintw(w,"%10s:Sem Count = %04x",e->name,e->EventCount);
}

void IdleTask(void *arg)  __attribute__ ( ( noreturn ) );
void IdleTask(void *arg)
{
	WINDOW *win;
	extern unsigned int FreeMallocSpace(void);
	unsigned char uC;
	ECB *sem;
	time_t timedata;
	struct tm *tod;

	sem = NewSemaphore(0,SEMAPHORE_MODE_TIMEOUT,"MTestTO");
	win = newwin(16,60,0,0,NULL,FG(RED) | BG(WHITE),WIN_BOXED);
	WinSetTitle(win, "Statistics");
	uC = MCUSR;
	MCUSR = 0;
	wprintw(win,"%s %s %s %s %s",(uC & BIT(JTRF))?"JT":"NA",(uC & BIT(WDRF))?"WD":"NA",(uC & BIT(BORF))?"BO":"NA",(uC & BIT(EXTRF))?"XT":"NA",(uC & BIT(PORF))?"PO":"NA");
	Idle = 0;
	while(1)
	{
		PendSemaphore(sem,200);
		wmove(win,0,0);
		TIME_LATCH_TIME = 0;	//latch time
		timedata = TIME_SECONDS;
		wprintw(win,"%02x:%02x:%02x  Seconds= %08lx",*((volatile char *)0xf062) & 0xff,*((volatile char *)0xf061) & 0xff,*((volatile char *)0xf060) & 0xff,timedata );
		wmove(win,1,0);
		wprintw(win,"Free Heap Space = %u  Free Stack Heap = %u",FreeMallocSpace(),StackHeapAvailiable());
		DisplayTask(2,win,&Tmmc);
		DisplayTask(3,win,&T2);
		DisplayTask(4,win,&TIdle);
		DisplayTask(5,win,&TM);
		DisplayTask(6,win,&TSch);
		DisplayTask(7,win,&TCom);
		DisplayTask(8,win,&TCmd);
		DisplayTask(9,win,&TEvnt);
		DisplaySem(11,win,&TimerSem);
		DisplaySem(12,win,sem);
		tod = gmtime( &timedata );
		wmove(win,13,0);
		wprintw(win,"%02d:%02d:%02d",tod->tm_hour,tod->tm_min,tod->tm_sec);
		++Idle;
	}
}

void EventTask(void *arg)  __attribute__ ( ( noreturn ) );
void EventTask(void *arg)
{
	WINDOW *win;
	MSG *msg;

	win = newwin(16,30,16,0,NULL,FG(BLUE) | BG(CYAN),WIN_BOXED);
	scrollok(win,1);
	WinSetTitle(win, "Events");
	while(1)
	{
		msg = MqGet(EventMSG);
		wprintw(win,"Message %d\n",msg->MsgCmd);
		/*
		switch(msg->MsgCmd)
		{
			case TE_EVENT1:
				break;
			case TE_EVENT2:
				break;
			case TE_EVENT3:
				break;
		}
		*/
		DeleteMSG(msg);
	}
}

int main()
{
	*((volatile char *)0x0f003) = 0x01;	//increment boot counter
	Disable();
	reset();
	OS_InitMalloc(0,&__end_of_heap);
	InitBlockAlloc(64,8);
	PortInit();
	OSInit();
	CioInit();
	Timer0Init();
	RInit();
	xRInit();
	CreateTask(&TEvnt,EventTask,384,10,"Evnts",NULL);
	Insert(&ActiveTasks,&TEvnt);
	CreateTask(&TSch,ScheduleTask,384,3,"Sched",NULL);
	Insert(&ActiveTasks,&TSch);
	CreateTask(&TIdle,IdleTask,384,2,"Idler",NULL);
	Insert(&ActiveTasks,&TIdle);
	CreateTask(&TM,MemTestTask,384,1,"MemTs",NULL);
	Insert(&ActiveTasks,&TM);
	CreateTask(&T2,TaskTwo,384,8,"T2tsk",(void *)0x1234);
	Insert(&ActiveTasks,&T2);
	CreateSemaphore(&TimerSem,0,SEMAPHORE_MODE_BLOCKING,"TmrSem");
	OneSecond = NewSemaphore(0,SEMAPHORE_MODE_BLOCKING,"OneSec");
	EventMSG = MqInit(32,"Event Msg");
	InitDDS();
	InitTimerServices();
	InitCommProtocol();
	spiInit();
	mmcInit();
	InitVGA();
	initscr();
	ExtInt4Init();
	StartOS();
	while(1);
	return 0;
}

