/*@A (C) 1992 Allen I. Holub                                                */
#include "video.h"
#include "termlib.h"

static void cpy_row ( int dest_row, int src_row, int left_col, int right_col	);
static void cpy_col ( int dest_col, int src_col, int top_row,  int bot_row	);
static void clr_col ( int col, int attrib, int top_row,  int bot_row   	);
static void clr_row ( int row, int attrib, int left_col, int right_col		);

static	void cpy_row(int dest_row,int src_row,int left_col,int right_col )
{
    /* Copy all characters between left_col and right_col (inclusive)
     * from src_row to the equivalent position in dest_row.
     */

    CHARACTER  *s ;
    CHARACTER  *d ;

    d = & SCREEN[ dest_row ][ left_col ];
    s = & SCREEN[ src_row  ][ left_col ];

    while( left_col++ <= right_col )
		*d++ = *s++;
}

/*----------------------------------------------------------------------*/

static	void cpy_col(int dest_col,int src_col,int top_row,int bot_row )
{
    /* Copy all characters between top_row and bot_row (inclusive)
     * from src_col to the equivalent position in dest_col.
     */

    CHARACTER  *s = & SCREEN[ top_row ][ src_col  ];
    CHARACTER  *d = & SCREEN[ top_row ][ dest_col ];

    while( top_row++ <= bot_row )
    {
		*d = *s;
		d += NUMCOLS;
		s += NUMCOLS;
    }
}

/*----------------------------------------------------------------------*/

static	void clr_row(int row,int attrib,int left_col,int right_col )
{
    /* Clear all characters in the indicated row that are between left_col and
     * right_col (inclusive).
     */

    CHARACTER  *p = & SCREEN[ row ][ left_col ];

    while( left_col++ <= right_col )
    {
		(p  )->letter    = ' ';
		(p++)->attribute = attrib ;
    }
}

/*----------------------------------------------------------------------*/

static	void clr_col(int col,int attrib,int top_row,int bot_row )
{
    /* Clear all characters in the indicated column that are between top_row
     * and bot_row (inclusive).
     */

    CHARACTER  *p = & SCREEN[ top_row ][ col ];

    while( top_row++ <= bot_row )
    {
	    p->letter    = ' ';
	    p->attribute = attrib ;
	    p += NUMCOLS ;
    }
}

/*======================================================================*
 *            	     Externally accessible functions:			*
 *======================================================================*
 */

void dv_scroll_line (int x_left,int x_right,int y_top,int y_bottom,int dir,int attrib )
{
    /* Scroll the window located at:
     *
     * (y_top, x_left) 	+---------+
     *			|	  |
     *			|	  |
     *			+---------+ (y_bottom, x_right)
     *
     * Dir is one of:  'u', 'd', 'l', or 'r' for up, down, left, or right.
     * The cursor is not moved. The opened line is filled with space characters
     * having the indicated attribute.
     */

    int  i;

    if( dir == 'u' )
    {
		for( i = y_top; i < y_bottom ; i++ )
		    cpy_row( i, i+1, x_left, x_right );
		clr_row( y_bottom, attrib, x_left, x_right );
    }
    else if( dir == 'd' )
    {
		for( i = y_bottom; --i >= y_top ; )
	    	cpy_row( i+1, i, x_left, x_right );
		clr_row( y_top, attrib, x_left, x_right );
    }
    else if( dir == 'l' )
    {
		for( i = x_left; i < x_right; i++ )
	    	cpy_col( i, i+1, y_top, y_bottom );
		clr_col( x_right, attrib, y_top, y_bottom );
    }
    else /* dir == 'r' */
    {
		for( i = x_right; --i >= x_left ; )
	    	cpy_col( i+1, i, y_top, y_bottom );
		clr_col( x_left, attrib, y_top, y_bottom );
    }
}

/*----------------------------------------------------------------------*/

void dv_scroll(int x_left,int x_right,int y_top,int y_bottom,int amt,int attrib )
{
    /* Scroll the screen up or down by the indicated amount. Negative
     * amounts scroll down.
     */

    int dir = 'u';

    if( amt < 0 )
    {
		amt = -amt;
		dir = 'd' ;
    }
    while( --amt >= 0 )
		dv_scroll_line( x_left, x_right, y_top, y_bottom, dir, attrib );
}
