/************************************************************************************
** Driver for fpga based SPI port
**
** created July 24, 2008
** Jim Patchell
**
*************************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include "system.h"
#include <stdio.h>
#include <stdlib.h>
#include "xspi.h"
#include "task.h"

static ECB *XspiSem,*XspiBlock;
volatile int Int7Count,MaxCount;

void INT7_vect(void) __attribute__ ((naked));
void INT7_vect(void)
{
	SAVE_IRQ();
	++InterruptCount;
	++Int7Count;
	if(Int7Count > MaxCount)
		MaxCount = Int7Count;
	XSPI_CLEAR_IRQ();	//clear Xspi interrupt
	EIFR = BIT(INT7);
	PostSemaphore(XspiSem,0);	//send notification of transfer
	ExitInterrupt();
	RESTORE_IRQ();
}

int XspiTransfer(char *buff, int n)
{
	int i;

	PendSemaphore(XspiBlock,0);	//lock access to spi port
	for(i=0;i<n;++i)
		XSPI_DATA = buff[i];	//stuff data into fifo
	XSPI_STARTPORT();			//start transfer
	PendSemaphore(XspiSem,0);	//wait for transfer to complete
	for(i=0;i<n;++i)
		buff[i] = XSPI_DATA;	//transfer data back into buffer
	PostSemaphore(XspiBlock,0);	//unlock access to spi port
	return 0;
}
/*
int SpiTransfer(char *buff,int n)
{
	int i;
	
	for(i=0;i<n;++i)
	{
		SpiPack.b[SpiPack.head++] = buff[i];
		if(SpiPack.head == SpiPack.size) SpiPack.head = 0;
	}
	SpiPack.nCout = n;
	// enable SPI interrupt
	SPDR = SpiPack.b[SpiPack.tail++];
	if(SpiPack.tail == SpiPack.size) SpiPack.tail = 0;
	--SpiPack.nCout;
	SPCR |= BIT(SPIE);
	PendSemaphore(SpiPack.Sem,0);
	for(i=0;i<n;++i)
	{
		buff[i] = SpiPack.b[SpiPack.tail++];
		if(SpiPack.tail == SpiPack.size) SpiPack.tail = 0;
	}
	SpiPack.nCin = 0;
	return SpiPack.nCin;	
}
*/

int WaitXspiDone(void)
{
	return PendSemaphore(XspiSem,0);
}

void InitXspi(void)
{
	XspiSem = NewSemaphore(0,SEMAPHORE_MODE_BLOCKING,"XSPIsem");
	XspiBlock = NewSemaphore(1,SEMAPHORE_MODE_BLOCKING,"XSPIBlock");
	XspiSetBitRate(400000L);	//400K bits per second
	EIFR = BIT(INT7);
	EIMSK |= BIT(INT7);
	EIFR = BIT(INT7);
	XSPI_IRQEN();
}

void XspiSetBitRate(unsigned long Rate)
{
	unsigned long BR;

	BR = XSPI_CLOCK / Rate;
	BR--;	//subtract 1
	BR /= 2;	//divide by two
	XSPI_BITRATE = (unsigned char)BR;
}

unsigned char XspiTransferByte(unsigned char data)
{
	PendSemaphore(XspiBlock,0);	//lock access to spi port
	XSPI_DATA = data;	//stuff data into fifo
	XSPI_STARTPORT();			//start transfer
	PendSemaphore(XspiSem,0);	//wait for transfer to complete
	--Int7Count;
	data = XSPI_DATA;	//transfer data back into buffer
	PostSemaphore(XspiBlock,0);	//unlock access to spi port
	// return the received data
	return data;
}


