/****************************************************************
** This file contains the objects for creating an event trigger
**
** It creates an event based on a trigger variable, and the past
** history of that variable.
**
** The event is signaled by calling a callback function
**
** Created May 7, 2008
**
****************************************************************/

#ifndef EVENTTRIGGER__H
#define EVENTTRIGGER__H

#define EVENTTRIGGER_MODE_POS	1
#define EVENTTRIGGER_MODE_NEG	2

#define EVENTTRIGGER_MAXEVENTS	16

typedef struct {
	unsigned char Previous;		//history of past triggers
	unsigned char TrigMode;		//trigger mode
	void *arg;					//pointer to arguments to pass callback
	void (*callback)(char trig,void *arg);	//callback function
}EVTB;

extern EVTB *NewEventTrigger(unsigned char mode, void (*cb)(char,void*),void *arg);
extern void FreeEventTrigger(EVTB *pE);
extern int EventTrigger(EVTB *pE,unsigned char v);
extern void InitEventTriggers(int n);
extern int CheckEventTrigger(int e,int v);
#endif
