/*******************************************************************
** Device Driver for UARTS on ATmega640/1280/1281/2561
**
** Craeted July 10, 2007
** by Jim Patchell
**
** Written for WinAVR
**
********************************************************************/

#ifndef XRS232__H
#define XRS232__H

#include "task.h"

#define XRS232_N_PORTS			1
#define XRS232_DEF_BAUDRATE		115200L
#define XRS232_DEF_T_BUFF_SIZE	256
#define XRS232_DEF_R_BUFF_SIZE	256

#define XRS232_STAT_NREC			0
#define XRS232_STAT_TMT			1

#define XRS232_XIO_SETBAUD		0	/*!< XIO Set Baud Rate command	*/

#define XRS232_TSR_FIFO_FULL	(4)	/*!< Transmit Fifo Full Status Bit	*/
#define XRS232_TSR_FIFO_MT		(5)	/*!< Transmit Fifo Empty Status Bit	*/
#define XRS232_TSR_FIFO_CNT_MSK	(0xf)

#define XRS232_RSR_FIFO_CNT_MSK	(0xf)
#define XRS232_RSR_FIFO_FULL	(4)
#define XRS232_RSR_FIFO_MT		(5)
#define XRS232_RSR_OVRRUN		(6)
#define XRS232_RSR_FRAME		(7)

#define XRS232_IER_RECV			(0)
#define XRS232_IER_TXMT			(1)
#define XRS232_IER_TXDONE		(2)

#define XRS232_IFR_RECV			(0)
#define XRS232_IFR_TXMT			(1)
#define XRS232_IFR_TXDONE		(2)

#define XRS232_IVR_RECV			0
#define XRS232_IVR_TXMT			1
#define XRS232_IVR_TXDONE		2

typedef struct {
	volatile unsigned char *pUDR;	//uart data register
	volatile unsigned char *pIER;	//interrupt enable register
	volatile unsigned char *pIFR;	//interrupt flag register
	volatile unsigned char *pIVR;	//interrupt vector register
	volatile unsigned char *pRSR;	//recieve status register
	volatile unsigned char *pTSR;	//transmit status register
	volatile unsigned short *pUBRR;	//baud rate generator (16 bits)
}xRREG;

typedef struct {
	unsigned short head;
	unsigned short tail;
	unsigned short nChar;
	unsigned short size;
	ECB *pSem;
	char *buff;
}xIOREC;

typedef struct {
	IOREC Rx;
	IOREC Tx;
	xRREG *regs;
} xSERIAL;

extern int xRInit(void);

#endif
