/************************************************************************************
** Driver for fpga based SPI port
**
** created July 24, 2008
** Jim Patchell
**
*************************************************************************************/

#ifndef XSPI__H
#define XSPI__H

#define XSPI_DATA		*((volatile unsigned char *)0xf078)
#define XSPI_BITRATE	*((volatile unsigned char *)0xf079)
#define XSPI_STATUS		*((volatile unsigned char *)0xf07a)
#define XSPI_CONTROL	*((volatile unsigned char *)0xf07b)
#define XSPI_NRFIFO		*((volatile unsigned *)0xf07c)
#define XSPI_NTFIFO		*((volatile unsigned *)0xf07e)

#define XSPI_CLOCK		(50000000L)

//--------------------------------------
// bit defines for STATUS
//--------------------------------------

#define XSPI_STATUS_TFIFOMT			(0)
#define XSPI_STATUS_TFIFOFULL		(1)
#define XSPI_STATUS_RFIFOMT			(2)
#define XSPI_STATUS_RFIFOFULL		(3)
#define XSPI_STATUS_IRQ				(4)	//on write, clears irq bit

//--------------------------------------
// Bit defines for control reg
//--------------------------------------

#define XSPI_CTL_IRQEN				(0)
#define XSPI_CTL_STARTPORT			(1)
#define XSPI_CTL_CE					(2)


//---------------------------------------
// operations
//---------------------------------------

#define XSPI_CLEAR_IRQ()	XSPI_STATUS = BIT(XSPI_STATUS_IRQ)
#define XSPI_IRQEN()		XSPI_CONTROL |= BIT(XSPI_CTL_IRQEN)
#define XSPI_STARTPORT()	XSPI_CONTROL |= BIT(XSPI_CTL_STARTPORT)
#define XSPI_IRQDIS()		XSPI_CONTROL &= ~(BIT(XSPI_CTL_IRQEN))
#define XSPI_SET_CE()		XSPI_CONTROL |= BIT(XSPI_CTL_CE)
#define XSPI_CLR_CE()		XSPI_CONTROL &= ~(BIT(XSPI_CTL_CE))

extern void InitXspi(void);
extern void XspiSetBitRate(unsigned long Rate);
extern unsigned char XspiTransferByte(unsigned char data);
extern int XspiTransfer(char *buff, int n);

#endif
