/*******************************************************************************
** Message Queue Manager
** Created April 12, 2007
** Jim Patchell
*******************************************************************************/

#include "stdio.h"
#include "stdlib.h"
#include "task.h"
#include "MessageQueue.h"

static ECB *MqBlocker;
int MsgAllocCount = 0;

MESSAGE_QUEUE * MqInit(int size,char *name)
{
	char *buffer = malloc(sizeof(MESSAGE_QUEUE) + size * sizeof(MSG*) );
	MESSAGE_QUEUE *rv = (MESSAGE_QUEUE *)buffer;
	MSG **pM = (MSG **)(&buffer[sizeof(MESSAGE_QUEUE)]);
	PendSemaphore(MqBlocker,0);		//protect access
	rv->Head = 0;
	rv->Tail = 0;
	rv->nMsg = 0;
	rv->b = pM;
	rv->Size = size;
	rv->Sem = NewSemaphore(0,SEMAPHORE_MODE_BLOCKING,name);
	PostSemaphore(MqBlocker,0);		//release access
	return rv;
}

void MqPut(MESSAGE_QUEUE *mq,MSG *m)
{
	char sr;

	if(mq)	//are we live?
	{
		sr = Disable();		//enter critical section
		if(mq->nMsg < mq->Size)	//is there room?
		{
			mq->b[mq->Head++] = m;	//put message into queue
			if(mq->Head == mq->Size)
				mq->Head = 0;	//wrap head pointer
			mq->nMsg++;			//increment number of messages
			PostSemaphore(mq->Sem,0);
		}
		Enable(sr);			//exit critical section
	}
}

MSG *MqGet(MESSAGE_QUEUE *mq)
{
	char sr;
	MSG *retval = 0;

	if(mq)
	{
		PendSemaphore(mq->Sem,0);		//wait for message
		sr = Disable();				//endter critical section
		retval = mq->b[mq->Tail++];	//remove message from queue
		if(mq->Tail == mq->Size)	//are we at the end of the line
			mq->Tail = 0;			//wrap tail pointer
		mq->nMsg--;					//decrement number of messages in queue
		Enable(sr);
	}
	return retval;
}

MSG *NewMSG(int payloadSize)
{
	MSG *retval = 0;

	retval = (MSG *)malloc(sizeof(MSG) + payloadSize);
	MsgAllocCount++;
	retval->MaxPayload = payloadSize;
	retval->ReplyHandle = NULL;
	retval->PayloadSize = 0;
	retval->next = 0;
	return retval;
}

void DeleteMSG(MSG *m)
{
	//-------------------------------------
	//put message into message pool
	//-------------------------------------
	free(m);
}

void InitMessageQueueManager(void)
{
	MqBlocker = NewSemaphore(1,SEMAPHORE_MODE_BLOCKING,"MSG_BLOCKER");
}

