// BaudRate.cpp: implementation of the BaudRate class.
//
//////////////////////////////////////////////////////////////////////
//
// This is a utility class for doing things concerning baudrate
//
// Created June 7, 2007
// By Jim Patchell
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BaudRate.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

static int BRLUT[] = {CBR_300,CBR_1200,CBR_2400,CBR_4800,CBR_9600,CBR_14400,CBR_19200,CBR_38400,CBR_57600,CBR_115200,-1};

BaudRate::BaudRate()
{
	int i;

	//------------------------------
	// find limit of baud rate look
	// up table
	//------------------------------
	for(i=0;BRLUT[i] != -1;++i);
	m_MaxBRIndex = i;
}

BaudRate::~BaudRate()
{

}

int BaudRate::GetBaudRate(int BRIndex)
{
	//---------------------------------------------
	// Get the baudrate from the baud rate look up
	// table
	//
	// parameters:
	//	BRIndex......index into baudrate table
	//
	// return value:
	//	return baud rate on success
	//	return 0 on fail
	//---------------------------------------------
	int retval;

	if(BRIndex < m_MaxBRIndex)
		retval = BRLUT[BRIndex];
	else
		retval = 0;
	return retval;
}

int BaudRate::GetBaudRateIndex(int BR)
{
	//---------------------------------------------
	// This function takes a baudrate value as
	// define hopefully in winbase.h and translates
	// it into a baud rate index that is used
	// by the COverlappedSerial object to set
	// baudrate
	//
	// parameter:
	//	BR.........The real baudrate
	//
	// return value:
	//	-1 on failure (baudrate not in table)
	//	index into baud rate table on success
	//---------------------------------------------
	int i;
	int retval = -1;

	for(i=0;BRLUT[i] > 0;++i)
	{
		if(BR == BRLUT[i]) retval = i;
	}
	return retval;
}

char * BaudRate::GetComPortName(int ComPort)
{
	//---------------------------------------------
	// This fucntion generates the comm port name
	// parameter:
	//	ComPort....numeric value
	//				0 => COM1
	//				1 => COM2
	//				etc.
	//
	// return value:
	//	pointer to a static string that contains
	// the name of the com port
	//----------------------------------------------
	static char s[32];

	sprintf(s,"COM%d",ComPort+1);
	return s;
}
