// Buffer.cpp: implementation of the CBuffer class.
//
//////////////////////////////////////////////////////////////////////
//
// This class is a simple buffer used to store data
// The buffer is circular, but it will not overwrite data that
// has be previously written
// 
// Created June 20, 2007
// bu Jim Patchell
///////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Buffer.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBuffer::CBuffer(int size)
{
	//--------------------------------------------------
	// create buffer
	//
	// parameter:
	//	size......size of buffer to create
	//--------------------------------------------------
	m_size = size;
	m_buff = new char[size];	//alocate memory for buffer
	m_nChar = 0;
	m_tail = 0;
	m_head = 0;
	m_mode = CBUFFER_MODE_WRITE;	//put buffer into write mode
}

CBuffer::~CBuffer()
{
	delete[] m_buff;	//delete memory used by buffer
}

int CBuffer::Get()
{
	//------------------------------------------------------
	// get a character from the buffer
	//
	// return value:
	//	returns value currently pointed to
	//	On fail, returns -1
	//------------------------------------------------------
	int retval;

	if(m_nChar && (m_mode == CBUFFER_MODE_READ))	//can we read some data?
	{
		retval = (int)m_buff[m_tail++] & 0x0ff;	//get data from buffer
		if(m_tail == m_size) m_tail = 0;	//wrap tail pointer if needed
		m_nChar--;		//decrement number of characters is buffer
	}
	else
		retval = -1;	//invalid..no data in buffer
	return retval;
}

void CBuffer::Put(int c)
{
	//-------------------------------------------------------
	// Add a character to the buffer
	//
	// parameter:
	//	c.........data to add to buffer
	//-------------------------------------------------------
	
	//---------------------------------------
	// can we add a character to the buffer?
	//---------------------------------------
	if((m_nChar < m_size) && (m_mode == CBUFFER_MODE_WRITE))
	{
		m_buff[m_head++] = (char)c;	//add character to buffer
		if(m_head == m_size) m_head = 0; //wrap head pointer, if needed
		m_nChar++;	//increment number of characters
	}
}

int CBuffer::Status()
{
	return m_nChar;		//return number of characters in buffer
}



void CBuffer::ReadMode()
{
	m_mode = CBUFFER_MODE_READ;	//put buffer into read mode
}

void CBuffer::WriteMode()
{
	m_mode = CBUFFER_MODE_WRITE;	//put buffer into write mode
}

int CBuffer::GetMode()
{
	return m_mode;		//return current mode of buffer
}
