// Synth.cpp : implementation file
//

#include "stdafx.h"
#include "synthconsol.h"
#include "Synth.h"
#include <math.h>
#include "WindowMessages.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


static char *Notes[12] = {
	"C",
	"C#",
	"D",
	"D#",
	"E",
	"F",
	"F#",
	"G",
	"G#",
	"A",
	"A#",
	"B"
};

static void PitchDisplay(void *,char *s,int l,int v)
{
	//--------------------------------------------------
	// displays text value in a string
	//
	// parameter:
	//	p.........user defined parameter, dependent on callback function
	//	s.........points to the character array where text is going to be put
	//	l.........maximum size of character array where text is put
	//	v.........value to display...somehow
	//  v will have the value of 0 to 128, which will corespond
	//  to a note on the keyboard.
	//----------------------------------------------------
	int Octave = (v / 12)-4;
	int Note = v % 12;
	sprintf(s,"%s%d",Notes[Note],Octave);
}

/////////////////////////////////////////////////////////////////////////////
// CSynth dialog


CSynth::CSynth(CWnd* pParent /*=NULL*/)
	: CDialog(CSynth::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSynth)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_pEVparams = new EVparams[512];	//place to store params for 512 envelope generators YIKEs!
}


void CSynth::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSynth)
	DDX_Control(pDX, IDC_STATIC_FLT_RES, m_Static_Flt_Res);
	DDX_Control(pDX, IDC_STATIC_FLT_FREQ, m_Static_Flt_Freq);
	DDX_Control(pDX, IDC_SB_FLT_RESONANCE, m_SB_Flt_Res);
	DDX_Control(pDX, IDC_SB_FLT_FREQUENCY, m_SB_Flt_Freq);
	DDX_Control(pDX, IDC_STATIC_SYNTH_RELEASE2, m_Static_Release2);
	DDX_Control(pDX, IDC_STATIC_SUSTAIN2, m_Static_Sustain2);
	DDX_Control(pDX, IDC_STATIC_DECAY2, m_Static_Decay2);
	DDX_Control(pDX, IDC_STATIC_ATTACK2, m_Static_Attack2);
	DDX_Control(pDX, IDC_SB_SUSTAIN2, m_ScrollBar_Sustain2);
	DDX_Control(pDX, IDC_SB_RELEASE2, m_ScrollBar_Release2);
	DDX_Control(pDX, IDC_SB_DECAY2, m_ScrollBar_Decay2);
	DDX_Control(pDX, IDC_SB_ATTACK2, m_ScrollBar_Attack2);
	DDX_Control(pDX, IDC_CHECK_TRIGGER2, m_Check_Trigger2);
	DDX_Control(pDX, IDC_CHECK_TRIGGER, m_Check_Trigger);
	DDX_Control(pDX, IDC_STATIC_SYNTH_RELEASE, m_Static_Release);
	DDX_Control(pDX, IDC_STATIC_SUSTAIN, m_Static_Sustain);
	DDX_Control(pDX, IDC_STATIC_DECAY, m_Static_Decay);
	DDX_Control(pDX, IDC_STATIC_ATTACK, m_Static_Attack);
	DDX_Control(pDX, IDC_SB_RELEASE, m_ScrollBar_Release);
	DDX_Control(pDX, IDC_SB_SUSTAIN, m_ScrollBar_Sustain);
	DDX_Control(pDX, IDC_SB_DECAY, m_ScrollBar_Decay);
	DDX_Control(pDX, IDC_SB_ATTACK, m_ScrollBar_Attack);
	DDX_Control(pDX, IDC_STATIC_SYNTH_PITCH, m_Static_Pitch);
	DDX_Control(pDX, IDC_SB_SYNTH_PITCH, m_SB_Pitch);
	DDX_Control(pDX, IDC_SB_SYNTH_CHANNEL, m_Scrollbar_Channel);
	DDX_Control(pDX, IDC_STATIC_SYNTH_CHANNEL, m_Static_Channel);
	DDX_Control(pDX, IDC_STATIC_DB, m_Static_Debug);
	//}}AFX_DATA_MAP
}

#define WM_FILT_FREQ			WM_USER+111
#define WM_FILT_RES				WM_USER+112

BEGIN_MESSAGE_MAP(CSynth, CDialog)
	//{{AFX_MSG_MAP(CSynth)
	ON_BN_CLICKED(IDC_CHECK_TRIGGER, OnCheckTrigger)
	ON_BN_CLICKED(IDC_CHECK_TRIGGER2, OnCheckTrigger2)
	ON_BN_CLICKED(IDC_BUTTON_SYNTH_SU_ZERO_OSC, OnButtonSynthSuZeroOsc)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_PITCHNOTIFICATION,OnPitchChange)
	ON_MESSAGE(WM_CHANNELNOTIFICATION,OnChannelChange)
	ON_MESSAGE(WM_OSCFREQ,OnOscFreq)
	ON_MESSAGE(WM_EV1_ATTACK_NOT,OnAttack1)
	ON_MESSAGE(WM_EV1_DECAY_NOT,OnDecay1)
	ON_MESSAGE(WM_EV1_SUSTAIN_NOT,OnSustain1)
	ON_MESSAGE(WM_EV1_RELEASE_NOT,OnRelease1)
	ON_MESSAGE(WM_EV2_ATTACK_NOT,OnAttack2)
	ON_MESSAGE(WM_EV2_DECAY_NOT,OnDecay2)
	ON_MESSAGE(WM_EV2_SUSTAIN_NOT,OnSustain2)
	ON_MESSAGE(WM_EV2_RELEASE_NOT,OnRelease2)
	ON_MESSAGE(WM_FILT_FREQ,OnFltFreq)
	ON_MESSAGE(WM_FILT_RES,OnFltResonance)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSynth message handlers

BOOL CSynth::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	m_Static_Debug.InitStringFifo(7);
	//----------------------------------------------
	// setup Envelope bank lookup tables
	//----------------------------------------------

	m_EVbank[0].pAttack = &m_ScrollBar_Attack;
	m_EVbank[0].pDecay = &m_ScrollBar_Decay;
	m_EVbank[0].pSustain = &m_ScrollBar_Sustain;
	m_EVbank[0].pRelease = &m_ScrollBar_Release;
	m_EVbank[1].pAttack = &m_ScrollBar_Attack2;
	m_EVbank[1].pDecay = &m_ScrollBar_Decay2;
	m_EVbank[1].pSustain = &m_ScrollBar_Sustain2;
	m_EVbank[1].pRelease = &m_ScrollBar_Release2;

	//-------------------------------------------------
	// setup scroll bars
	//-------------------------------------------------
	m_Scrollbar_Channel.SetScrollRange(0,511);
	m_Scrollbar_Channel.SetDisplayText(&m_Static_Channel);
	m_Scrollbar_Channel.SetActionMessage(m_hWnd,WM_CHANNELNOTIFICATION);
	m_Scrollbar_Channel.SetScrollPos(0);
	
	m_SB_Pitch.SetScrollRange(0,127);
	m_SB_Pitch.SetDisplayText(&m_Static_Pitch);
	m_SB_Pitch.SetDisplayFormatFunction(PitchDisplay);
	m_SB_Pitch.SetActionMessage(m_hWnd,WM_PITCHNOTIFICATION);
	m_SB_Pitch.SetScrollPos(48);

	//----------------------------------------
	// setupt Envelope generator 1 controls
	//----------------------------------------

	m_ScrollBar_Attack.SetScrollRange(0,2047);
	m_ScrollBar_Attack.SetDisplayText(&m_Static_Attack);
	m_ScrollBar_Attack.SetActionMessage(m_hWnd,WM_EV1_ATTACK_NOT);
	m_ScrollBar_Attack.SetScrollPos(0);

	m_ScrollBar_Decay.SetScrollRange(0,2047);
	m_ScrollBar_Decay.SetDisplayText(&m_Static_Decay);
	m_ScrollBar_Decay.SetActionMessage(m_hWnd,WM_EV1_DECAY_NOT);
	m_ScrollBar_Decay.SetScrollPos(0);

	m_ScrollBar_Sustain.SetScrollRange(0,2047);
	m_ScrollBar_Sustain.SetDisplayText(&m_Static_Sustain);
	m_ScrollBar_Sustain.SetActionMessage(m_hWnd,WM_EV1_SUSTAIN_NOT);
	m_ScrollBar_Sustain.SetScrollPos(0);

	m_ScrollBar_Release.SetScrollRange(0,2047);
	m_ScrollBar_Release.SetDisplayText(&m_Static_Release);
	m_ScrollBar_Release.SetActionMessage(m_hWnd,WM_EV1_RELEASE_NOT);
	m_ScrollBar_Release.SetScrollPos(0);

	//----------------------------------------
	// setupt Envelope generator 2 controls
	//----------------------------------------

	m_ScrollBar_Attack2.SetScrollRange(0,2047);
	m_ScrollBar_Attack2.SetDisplayText(&m_Static_Attack2);
	m_ScrollBar_Attack2.SetActionMessage(m_hWnd,WM_EV2_ATTACK_NOT);
	m_ScrollBar_Attack2.SetScrollPos(0);

	m_ScrollBar_Decay2.SetScrollRange(0,2047);
	m_ScrollBar_Decay2.SetDisplayText(&m_Static_Decay2);
	m_ScrollBar_Decay2.SetActionMessage(m_hWnd,WM_EV2_DECAY_NOT);
	m_ScrollBar_Decay2.SetScrollPos(0);

	m_ScrollBar_Sustain2.SetScrollRange(0,2047);
	m_ScrollBar_Sustain2.SetDisplayText(&m_Static_Sustain2);
	m_ScrollBar_Sustain2.SetActionMessage(m_hWnd,WM_EV2_SUSTAIN_NOT);
	m_ScrollBar_Sustain2.SetScrollPos(0);

	m_ScrollBar_Release2.SetScrollRange(0,2047);
	m_ScrollBar_Release2.SetDisplayText(&m_Static_Release2);
	m_ScrollBar_Release2.SetActionMessage(m_hWnd,WM_EV2_RELEASE_NOT);
	m_ScrollBar_Release2.SetScrollPos(0);

	//--------------------------------------------
	// Setup Filter controls
	//--------------------------------------------

	m_SB_Flt_Freq.SetScrollRange(0,2047);
	m_SB_Flt_Freq.SetDisplayText(&m_Static_Flt_Freq);
	m_SB_Flt_Freq.SetActionMessage(m_hWnd,WM_FILT_FREQ);
	m_SB_Flt_Freq.SetScrollPos(0);

	m_SB_Flt_Res.SetScrollRange(0,2047);
	m_SB_Flt_Res.SetDisplayText(&m_Static_Flt_Res);
	m_SB_Flt_Res.SetActionMessage(m_hWnd,WM_FILT_RES);
	m_SB_Flt_Res.SetScrollPos(0);


	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CSynth::OnOK() 
{
	// TODO: Add extra validation here
	CWnd *wC;

	wC = GetFocus();					//get handle of control that generated message
	int id = wC->GetDlgCtrlID();		//get ID of control
	switch(id)							//parse the id
	{
		case IDOK:
			CDialog::OnOK();
			break;
		default:
			wC = GetNextDlgTabItem(wC);	//reset the focus
			wC->SetFocus();
			break;
	}
	
}

unsigned long CSynth::PitchToFrequency(int pitch)
{
	double NotePitch;
	double y;
	long retval;

	y = (double)pitch/12.0;
	NotePitch = ((16.35159783) / 16.0) * pow(2.0,y);
	retval = (long)((NotePitch / 96000.0) * 4294967296.0);
	char *s = new char[256];
	sprintf(s,"%lf  %d  %lf %lf %08lx",NotePitch,pitch,pow(2.0,y),y,retval);
	m_Static_Debug.SetWindowText(s);
	delete[] s;
	return retval;
}

int CSynth::FrequencyToPitch(long Freq)
{
	int retval=0;
	double NotePitch;

	if(Freq > 0)
	{
		NotePitch = ( (double) Freq * 96000.0) / 4294967296.0;
		double Note = 12.0 * logb(NotePitch / (16.35159783 / 16.0),2.0);
		double frac = Note - floor(Note);
		if(frac >= 0.5)
			Note = ceil(Note);
		else
			Note = floor(Note);
		retval = (int)Note;
		char *s = new char[256];
		sprintf(s,"%lf%08lx N=%lf",NotePitch,Freq,Note);
		m_Static_Debug.SetWindowText(s);
		delete[] s;
	}
	return retval;
}

double CSynth::logb(double x, double b)
{
	double retval;
	retval = log(x) / log(b);
	return retval;
}

void CSynth::OnCheckTrigger() 
{
	// TODO: Add your control notification handler code here
	int v = m_Check_Trigger.GetCheck();
	char *s = new char[256];
	sprintf(s,"Trigger 1 = %d",v);
	m_Static_Debug.SetWindowText(s);
	m_pSynthPort->SendSetEvTrigger(0,m_Scrollbar_Channel.pos,v);
	delete[] s;
}

void CSynth::OnCheckTrigger2() 
{
	// TODO: Add your control notification handler code here
	
	int v = m_Check_Trigger2.GetCheck();
	char *s = new char[256];
	sprintf(s,"Trigger 2 = %d",v);
	m_Static_Debug.SetWindowText(s);
	m_pSynthPort->SendSetEvTrigger(0,m_Scrollbar_Channel.pos,v);
	delete[] s;
}

LRESULT CSynth::OnAttack1(WPARAM w, LPARAM l)
{
	char *s = new char[256];
	sprintf(s,"EV1 Attack = %d",ScaleTimeConstant(w));
	m_Static_Debug.SetWindowText(s);
	delete[] s;
	EVparams *pEvP = new EVparams;
	InitEvParams(&m_EVbank[0],pEvP);
	m_pSynthPort->SendSynthSetEv(0,m_Scrollbar_Channel.pos,pEvP);
	delete pEvP;
	return 0;
}

LRESULT CSynth::OnAttack2(WPARAM w, LPARAM l)
{
	char *s = new char[256];
	sprintf(s,"EV2 Attack = %d",w);
	m_Static_Debug.SetWindowText(s);
	delete[] s;
	return 0;
}

LRESULT CSynth::OnDecay1(WPARAM w, LPARAM l)
{
	char *s = new char[256];
	sprintf(s,"EV1 Decay = %d",w);
	m_Static_Debug.SetWindowText(s);
	delete[] s;
	EVparams *pEvP = new EVparams;
	InitEvParams(&m_EVbank[0],pEvP);
	m_pSynthPort->SendSynthSetEv(0,m_Scrollbar_Channel.pos,pEvP);
	delete pEvP;
	return 0;
}

LRESULT CSynth::OnDecay2(WPARAM w, LPARAM l)
{
	char *s = new char[256];
	sprintf(s,"EV2 Decay = %d",w);
	m_Static_Debug.SetWindowText(s);
	delete[] s;
	return 0;
}

LRESULT CSynth::OnSustain1(WPARAM w, LPARAM l)
{
	char *s = new char[256];
	sprintf(s,"EV1 Sustain = %d",w);
	m_Static_Debug.SetWindowText(s);
	delete[] s;
	EVparams *pEvP = new EVparams;
	InitEvParams(&m_EVbank[0],pEvP);
	m_pSynthPort->SendSynthSetEv(0,m_Scrollbar_Channel.pos,pEvP);
	delete pEvP;
	return 0;
}

LRESULT CSynth::OnSustain2(WPARAM w, LPARAM l)
{
	char *s = new char[256];
	sprintf(s,"EV2 Sustain = %d",w);
	m_Static_Debug.SetWindowText(s);
	delete[] s;
	return 0;
}

LRESULT CSynth::OnRelease1(WPARAM w, LPARAM l)
{
	char *s = new char[256];
	sprintf(s,"EV1 Release = %d",w);
	m_Static_Debug.SetWindowText(s);
	delete[] s;
	EVparams *pEvP = new EVparams;
	InitEvParams(&m_EVbank[0],pEvP);
	m_pSynthPort->SendSynthSetEv(0,m_Scrollbar_Channel.pos,pEvP);
	delete pEvP;
	return 0;
}

LRESULT CSynth::OnRelease2(WPARAM w, LPARAM l)
{
	char *s = new char[256];
	sprintf(s,"EV2 Release = %d",w);
	m_Static_Debug.SetWindowText(s);
	delete[] s;
	return 0;
}

LRESULT CSynth::OnFltFreq(WPARAM w, LPARAM l)
{
	char *s = new char[256];
	sprintf(s,"Filt Freq = %d",w);
	m_Static_Debug.SetWindowText(s);
	delete[] s;
	return 0;
}

LRESULT CSynth::OnFltResonance(WPARAM w, LPARAM l)
{
	char *s = new char[256];
	sprintf(s,"Filt Resonance = %d",w);
	m_Static_Debug.SetWindowText(s);
	delete[] s;
	return 0;
}

LRESULT CSynth::OnPitchChange(WPARAM wparam, LPARAM lparam)
{
	long pitch;
	int channel;
	channel = m_Scrollbar_Channel.pos;
	pitch = PitchToFrequency(wparam);
	m_pSynthPort->SendSynthSetOscFreq(channel,pitch);
	return 0;
}

LRESULT CSynth::OnChannelChange(WPARAM wparam, LPARAM lparam)
{
	m_pSynthPort->SendSynthGetOscFreq(wparam);
	return 0;
}

LRESULT CSynth::OnOscFreq(WPARAM channel, LPARAM freq)
{
	int N;

	N = FrequencyToPitch(freq);
	m_SB_Pitch.UpdateScrollPos(N);
	return 0;
}

void CSynth::FillInEVparam(int b,EVparams *pEV)
{
	pEV->Attack = m_EVbank[b].pAttack->pos;
	pEV->Decay = m_EVbank[b].pDecay->pos;
	pEV->Sustain = m_EVbank[b].pSustain->pos;
	pEV->Release = m_EVbank[b].pRelease->pos;
}

unsigned short CSynth::ScaleTimeConstant(int v)
{
	double dV = (double)v * 5.417850332e-3;
	dV = exp(dV);
	return (unsigned short)dV;
}

void CSynth::InitEvParams(EVbank *pEvB, EVparams *pEvP)
{
	pEvP->Attack = ScaleTimeConstant(pEvB->pAttack->pos);
	pEvP->Decay = ScaleTimeConstant(pEvB->pDecay->pos);
	pEvP->Sustain = 32 * pEvB->pSustain->pos;
	pEvP->Release = ScaleTimeConstant(pEvB->pRelease->pos);
}

void CSynth::OnButtonSynthSuZeroOsc() 
{
	// TODO: Add your control notification handler code here
	int channel;
	channel = m_Scrollbar_Channel.pos;
	m_pSynthPort->SendSynthSetOscFreq(channel,0);	
}
