// SynthPort.h: interface for the CSynthPort class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SYNTHPORT_H__6DA1BAA6_E04D_49BA_B6B0_589FC9757D08__INCLUDED_)
#define AFX_SYNTHPORT_H__6DA1BAA6_E04D_49BA_B6B0_589FC9757D08__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MSG.h"
#include "OverlappedSerial.h"
#include "EVparams.h"

//----------------------------------------------------------------------------
// misc defines
//----------------------------------------------------------------------------

#define LOW(x)		((x) & 0xff)
#define MIDLO(x)	(((x) >> 8) & 0x0ff)
#define MIDHI(x)	(((x) >> 16) & 0x0ff)
#define HIGH(x)		(((x) >> 24) & 0x0ff)

#define MON_STATE_IDLE			0
#define MON_STATE_COLLECT_SIZE	1
#define MON_STATE_COLLECTCMD	2
#define MON_STATE_COLLECTDATA	3

//------------------------------------------------------------------------------
// Message Definitions
//------------------------------------------------------------------------------

#define MM_OUT_ACK		1		//ACK....this is sent when command is executed
#define MM_OUT_NAK		2		//NAK....this is sent if there is a problem with command

#define MM_TEST			1
#define MM_SYNTH_EN				2
#define MM_ARM_LA				3
#define MM_READ_LA				4
#define MM_SET_LA_TRIG			5
#define MM_GET_LA_TRIG			6
#define MM_SET_OSC_FREQ			7
#define MM_GET_OSC_FREQ			8
#define MM_SET_EV				9
#define MM_GET_EV				10
#define MM_SET_FLT_FREQ			11
#define MM_GET_FLT_FREQ			12
#define MM_SET_FLT_Q			13
#define MM_GET_FLT_Q			14
#define MM_RESET_LA				15
#define MM_STATUS_LA			16
#define MM_SETQUAL_LA			17
#define MM_GETQUAL_LA			18
#define MM_PEEK					19
#define MM_SINGLESTEP			20
#define MM_GETDEBUG				21
#define MM_EV_TRIGGER			22
#define MM_EV_STATUS			23
#define MM_SET_SSCHANNEL		24
#define MM_GET_SSCHANNEL		25

/*!
 * Logic analyzer structures
 */

typedef struct _la_status LA_STATUS;

struct _la_status {
	unsigned short WordsWritten;
	unsigned short BytesRead;
	char State;
};

class CSynthPort  
{
	int m_ParseCompletMessage;
	HWND m_hWnd;
public:
	AMSG * pReleaseBuff;
	AMSG * msg;
	short index;
	union cvrt{
		char cV[2];
		short sV;
	}count,Cmd;
	AMSG * pBufferList;
	int m_NRelease;
	int m_NBuffers;
	int State;
	short MaxPayload;
	CMsgQueue * m_pMQ;
	char * IncommingPayload;
	COverlappedSerial * pComP;
public:
	void SendSingleStepGetChannel(void);
	void SendSingleStepSetChannel(unsigned short Ch);
	void SendGetEvStatus(int EvBank,int EvUnit);
	void SendSetEvTrigger(int EvBank,int EvUnit,int v);
	void SendGetDebugData(void);
	void SendSingleStep(void);
	void SendPeek(unsigned short adr);
	void SendLaGetQualifier(void);
	void SendLaSetQualifier(unsigned short Qaul);
	void SendLaStatus(void);
	void SendLaReset(void);
	void SendLAReadData(int n);
	void SendLAarm(void);
	void SendSynthGetFiltQ(int n);
	void SendSynthSetFiltQ(int n, unsigned short Q);
	void SendSynthGetFiltFreq(int n);
	void SendSynthSetFiltFreq(int n, unsigned short f);
	void SendSynthGetEV(int eg,int n);
	void SendSynthSetEv(int eg, int n,EVparams *pEV);
	void SendSynthGetOscFreq(int n);
	void SendSynthSetOscFreq(int n,unsigned long f);
	void SendSynthEnable(int enable);
	void SendLATrigger(char *pW);
	void SendTest(char *data);
	void SetMessageQueue(CMsgQueue *pMSQ);
	void SetParseCompleteMessage(HWND nhWnd, int msg);
	void SetComPort(COverlappedSerial *pSP);
	int ParseIncomingStream(int c);
	CSynthPort();
	virtual ~CSynthPort();

};

#endif // !defined(AFX_SYNTHPORT_H__6DA1BAA6_E04D_49BA_B6B0_589FC9757D08__INCLUDED_)
