`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company:  Zacca Design
// Engineer: Jim Patchell
// 
// Create Date:    20:13:55 04/21/2008 
// Design Name:
// Module Name:    Debug 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description:
//		This hardware is used to record all reads and writes until a trigger event happens, at
//	which time a port mortom can be done to determine just what was going on at the time of
// the crash...maybe 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: Copyright (c) 2008 by James Patchell
//		This file is in the public domain.  The user of this file takes all responibilities
// for its use.  More than likely, this code doesn't work correctly...it is up to you
// to make sure it does.
//
// And, if you are a professor in India, and one of your students was so stupid
// as to not remove this notice from one of his/her assignments...
// ...please make sure he or she fails the course.
//
//////////////////////////////////////////////////////////////////////////////////
module Debug(clk, reset,la_in,la_wd, din, dout, rd, wd, A);
    input clk;				//system clock
    input reset;			//system reset
	 input [31:0] la_in;	//intput to "logic analyzer"
	 input la_wd;			//write strobe to write logic data into ram
    input [7:0] din;	//data from AVR
    output [7:0] dout;	//data to AVR
    input rd;				//read data strobe from AVR
    input wd;				//write data strobe from AVR
	 input [3:0] A;		//address bus from AVR

	reg [7:0] dout;
	
	//--------------------------------------------------
	// Internal Register Defines
	//--------------------------------------------------
	
	parameter LA_REG_READADR_LO		=		4'b0000;
	parameter LA_REG_READADR_HI		=		4'b0001;
	parameter LA_REG_WRITEADR_LO		=		4'b0010;
	parameter LA_REG_WRITEADR_HI		=		4'b0011;
	parameter LA_REG_RAM					=		4'b0100;		//port to access current ram address
	parameter LA_TRIG0					=		4'b0110;
	parameter LA_TRIG1					=		4'b0111;
	parameter LA_TRIG2					=		4'b1000;
	parameter LA_TRIG3					=		4'b1001;
	parameter LA_TRIG4					=		4'b1010;
	parameter LA_TRIG5					=		4'b1011;
	parameter LA_TRIG6					=		4'b1100;
	parameter LA_TRIG7					=		4'b1101;
	parameter LA_REG_STATUS				=		4'b1111;		//read only status register
	parameter LA_REG_CONTROL			=		4'b1111;		//write only...control functions
	
	parameter LA_CTL_RESET_READADR	=		8'b00000001;	//reset read address register
	parameter LA_CTL_RESET_WRITEADR	=		8'b00000010;	//reset write address register
	parameter LA_CTL_ARM					=		8'b00000100;	//arm logic analyzer
	
	parameter LA_STATE_IDLE				=		2'b00;			//trigger sequencer IDLE state
	parameter LA_STATE_ARMED			=		2'b01;			//trigger sequencer ARMED state (ready to trigger)
	parameter LA_STATE_TRIGGERED		=		2'b10;			//trigger sequencer TRIGGERED state (recording data)
	parameter LA_STATE_DONE				=		2'b11;			//trigger sequencer DONE state (buffer is full)
	
	reg [31:0] la_in_p;
	reg la_wd_p;
	//---------------------------------------------------
	// Write Data controller
	//---------------------------------------------------
	
	reg [9:0] WriteAddress;
	reg wdWriteAddressLO,wdWriteAddressHI;
	reg clrWriteAddress;
	wire incWriteAddress;
	reg armLA;
	//----------------------------------------------------
	// Read Data controller
	//----------------------------------------------------
	
	reg [11:0] ReadAddress;
	reg wdReadAddressLO,wdReadAddressHI;
	reg clrReadAddress;
	reg incReadAddress;
	wire[7:0] dRam1,dRam2;
	reg [7:0] dRam;
	reg wdRam1,wdRam2;
	reg rdRam;
	
	//-----------------------------------------------------
	// Trigger logic
	//-----------------------------------------------------
	
	reg [63:0] Trigger;	//trigger control
	wire [31:0]TC;			//trigger compare bits
	reg TC_out;				//output of trigger compare
	reg [1:0] CS,NS;		//state variables for trigger sequencer
	reg la_wd_triggered;
	
	always @ (posedge clk) begin
		la_wd_p <= la_wd;
		la_in_p <= la_in;
	end
	//-----------------------------------------------------
	// address decoding logic
	//-----------------------------------------------------
	
	always @ (A or wd)
		if((A == LA_REG_READADR_LO) && wd)
			wdReadAddressLO = 1;
		else
			wdReadAddressLO = 0;
			
	always @ (A or wd)
		if((A == LA_REG_READADR_HI) && wd)
			wdReadAddressHI = 1;
		else
			wdReadAddressHI = 0;
			
	always @ (A or wd)
		if((A == LA_REG_WRITEADR_LO) && wd)
			wdWriteAddressLO = 1;
		else
			wdWriteAddressLO = 0;
		
	always @ (A or wd)
		if((A == LA_REG_WRITEADR_HI) && wd)
			wdWriteAddressHI = 1;
		else
			wdWriteAddressHI = 0;

	always @ (A or wd or ReadAddress)
		if((A == LA_REG_RAM) && wd && !ReadAddress[0])
			wdRam1 = 1;
		else
			wdRam1 = 0;

	always @ (A or wd or ReadAddress)
		if((A == LA_REG_RAM) && wd && ReadAddress[0])
			wdRam2 = 1;
		else
			wdRam2 = 0;

	always @ (A or rd)
		if((A == LA_REG_RAM) && rd)
			rdRam = 1;
		else
			rdRam = 0;

	always @ (A or wd or din)
		if((A == LA_REG_CONTROL) && wd && din[0])
			clrReadAddress = 1;
		else
			clrReadAddress = 0;

	always @ (A or wd or din)
		if((A == LA_REG_CONTROL) && wd && din[1])
			clrWriteAddress = 1;
		else
			clrWriteAddress = 0;

	always @ (A or wd or din)
		if((A == LA_REG_CONTROL) && wd && din[2])
			armLA = 1;
		else
			armLA = 0;
			
	//------------------------------------------------------
	// Read Data Controller
	// This is the port that is interfaced up to the AVR
	// despite the fact that it is called the read data
	// controller, this port can be used to do both reads
	// and writes.
	//------------------------------------------------------
	
	always @(posedge clk or posedge reset) begin
		if(reset)
			ReadAddress <= 0;
		else begin
			if(wdReadAddressLO)
				ReadAddress[7:0] <= din;
			else if (wdReadAddressHI)
				ReadAddress[11:8] <= din[3:0];
			else if (clrReadAddress)
				ReadAddress <= 0;
			else if (incReadAddress)
				ReadAddress <= ReadAddress + 1;
		end
	end

	always @ (posedge clk) begin
		if(rdRam || wdRam1 || wdRam2)
			incReadAddress = 1;
		else
			incReadAddress = 0;
	end
	
	//-------------------------------------------------------
	// Write Data Controller
	//
	// This is the address counter that controls where
	// the next chunk of data from the logic analyzer input
	// port is going to be written.  This register should be
	// set up before the logic analyzer is armed.
	//-------------------------------------------------------
	
	assign incWriteAddress = la_wd_triggered;
	
	always @ (posedge clk or posedge reset) begin
		if(reset)
			WriteAddress <= 0;
		else begin
			if(wdWriteAddressLO)
				WriteAddress[7:0] <= din;
			else if (wdWriteAddressHI)
				WriteAddress[9:8] <= din[1:0];
			else if( clrWriteAddress)
				WriteAddress <= 0;
			else if (incWriteAddress)
				WriteAddress <= WriteAddress + 1;
		end
	end

	//-------------------------------------------------------
	// Read data back to AVR
	//-------------------------------------------------------
	always @ (ReadAddress or dRam2 or dRam1)
		if(ReadAddress[0])
			dRam = dRam2;
		else
			dRam = dRam1;
		
	always @ (A or ReadAddress or WriteAddress or dRam or Trigger or CS) begin
		case(A)
			LA_REG_READADR_LO: dout = ReadAddress[7:0];	//ram read address register
			LA_REG_READADR_HI: dout = {4'b0000,ReadAddress[11:8]};
			LA_REG_WRITEADR_LO: dout = WriteAddress[7:0];	//ram write address register
			LA_REG_WRITEADR_HI: dout = {6'b000000,WriteAddress[9:8]};
			LA_REG_RAM: dout = dRam;			//read out ram
			LA_TRIG0:dout = Trigger[7:0];		//read out trigger register
			LA_TRIG1:dout = Trigger[15:8];
			LA_TRIG2:dout = Trigger[23:16];
			LA_TRIG3:dout = Trigger[31:24];
			LA_TRIG4:dout = Trigger[39:32];
			LA_TRIG5:dout = Trigger[47:40];
			LA_TRIG6:dout = Trigger[55:48];
			LA_TRIG7:dout = Trigger[63:56];
			LA_REG_STATUS:dout = {6'b000000,CS};	//read out status register
			default: dout = 8'h9a;
		endcase
	end
	
	//----------------------------------------------------------
	// Trigger
	//----------------------------------------------------------
	
	always @ (posedge clk or posedge reset) begin
		if(reset)
			Trigger <= 0;
		else begin
			if((A == LA_TRIG0) && wd)
				Trigger[7:0] <= din;
			else if ((A == LA_TRIG1) && wd)
				Trigger[15:8] <= din;
			else if ((A == LA_TRIG2) && wd)
				Trigger[23:16] <= din;
			else if ((A == LA_TRIG3) && wd)
				Trigger[31:24] <= din;
			else if ((A == LA_TRIG4) && wd)
				Trigger[39:32] <= din;
			else if ((A == LA_TRIG5) && wd)
				Trigger[47:40] <= din;
			else if ((A == LA_TRIG6) && wd)
				Trigger[55:48] <= din;
			else if ((A == LA_TRIG7) && wd)
				Trigger[63:56] <= din;
		end
	end
	
TrigCompare TrigCmp0 (
    .T(Trigger[1:0]), 
    .I(la_in_p[0]), 
    .O(TC[0])
    );

TrigCompare TrigCmp1 (
    .T(Trigger[3:2]), 
    .I(la_in_p[1]), 
    .O(TC[1])
    );

TrigCompare TrigCmp2 (
    .T(Trigger[5:4]), 
    .I(la_in_p[2]), 
    .O(TC[2])
    );

TrigCompare TrigCmp3 (
    .T(Trigger[7:6]), 
    .I(la_in_p[3]), 
    .O(TC[3])
    );

TrigCompare TrigCmp4 (
    .T(Trigger[9:8]), 
    .I(la_in_p[4]), 
    .O(TC[4])
    );

TrigCompare TrigCmp5 (
    .T(Trigger[11:10]), 
    .I(la_in_p[5]), 
    .O(TC[5])
    );

TrigCompare TrigCmp6 (
    .T(Trigger[13:12]), 
    .I(la_in_p[6]), 
    .O(TC[6])
    );

TrigCompare TrigCmp7 (
    .T(Trigger[15:14]), 
    .I(la_in_p[7]), 
    .O(TC[7])
    );

TrigCompare TrigCmp8 (
    .T(Trigger[17:16]), 
    .I(la_in_p[8]), 
    .O(TC[8])
    );

TrigCompare TrigCmp9 (
    .T(Trigger[19:18]), 
    .I(la_in_p[9]), 
    .O(TC[9])
    );

TrigCompare TrigCmp10 (
    .T(Trigger[21:20]), 
    .I(la_in_p[10]), 
    .O(TC[10])
    );

TrigCompare TrigCmp11 (
    .T(Trigger[23:22]), 
    .I(la_in_p[11]), 
    .O(TC[11])
    );

TrigCompare TrigCmp12 (
    .T(Trigger[25:24]), 
    .I(la_in_p[12]), 
    .O(TC[12])
    );

TrigCompare TrigCmp13 (
    .T(Trigger[27:26]), 
    .I(la_in_p[13]), 
    .O(TC[13])
    );

TrigCompare TrigCmp14 (
    .T(Trigger[29:28]), 
    .I(la_in_p[14]), 
    .O(TC[14])
    );

TrigCompare TrigCmp15 (
    .T(Trigger[31:30]), 
    .I(la_in_p[15]), 
    .O(TC[15])
    );

TrigCompare TrigCmp16 (
    .T(Trigger[33:32]), 
    .I(la_in_p[16]), 
    .O(TC[16])
    );

TrigCompare TrigCmp17 (
    .T(Trigger[35:34]), 
    .I(la_in_p[17]), 
    .O(TC[17])
    );

TrigCompare TrigCmp18 (
    .T(Trigger[37:36]), 
    .I(la_in_p[18]), 
    .O(TC[18])
    );

TrigCompare TrigCmp19 (
    .T(Trigger[39:38]), 
    .I(la_in_p[19]), 
    .O(TC[19])
    );

TrigCompare TrigCmp20 (
    .T(Trigger[41:40]), 
    .I(la_in_p[20]), 
    .O(TC[20])
    );

TrigCompare TrigCmp21 (
    .T(Trigger[43:42]), 
    .I(la_in_p[21]), 
    .O(TC[21])
    );

TrigCompare TrigCmp22 (
    .T(Trigger[45:44]), 
    .I(la_in_p[22]), 
    .O(TC[22])
    );

TrigCompare TrigCmp23 (
    .T(Trigger[47:46]), 
    .I(la_in_p[23]), 
    .O(TC[23])
    );

TrigCompare TrigCmp24 (
    .T(Trigger[49:48]), 
    .I(la_in_p[24]), 
    .O(TC[24])
    );

TrigCompare TrigCmp25 (
    .T(Trigger[51:50]), 
    .I(la_in_p[25]), 
    .O(TC[25])
    );

TrigCompare TrigCmp26 (
    .T(Trigger[53:52]), 
    .I(la_in_p[26]), 
    .O(TC[26])
    );

TrigCompare TrigCmp27 (
    .T(Trigger[55:54]), 
    .I(la_in_p[27]), 
    .O(TC[27])
    );

TrigCompare TrigCmp28 (
    .T(Trigger[57:56]), 
    .I(la_in_p[28]), 
    .O(TC[28])
    );

TrigCompare TrigCmp29 (
    .T(Trigger[59:58]), 
    .I(la_in_p[29]), 
    .O(TC[29])
    );

TrigCompare TrigCmp30 (
    .T(Trigger[61:60]), 
    .I(la_in_p[30]), 
    .O(TC[30])
    );

TrigCompare TrigCmp31 (
    .T(Trigger[63:62]), 
    .I(la_in_p[31]), 
    .O(TC[31])
    );

	always @ (TC)
		if(TC[0] && TC[1] && TC[2] && TC[3] && TC[4] && TC[5] && TC[6] && TC[7]
		&& TC[8] && TC[9] && TC[10] && TC[11] && TC[12] && TC[13] && TC[14] && TC[15]
		&& TC[16] && TC[17] && TC[18] && TC[19] && TC[20] && TC[21] && TC[22] && TC[23]
		&& TC[24] && TC[25] && TC[26] && TC[27] && TC[28] && TC[29] && TC[30] && TC[31])
			TC_out = 1;
		else
			TC_out = 0;
			
	always @ (posedge clk or posedge reset)
		if(reset)
			CS <= LA_STATE_IDLE;
		else
			CS <= NS;
			
	always @ (TC_out or CS or armLA or WriteAddress or la_wd_p) begin
		case(CS)
			LA_STATE_IDLE: begin			//trigger sequencer IDLE state
				if(armLA)
					NS = LA_STATE_ARMED;
				else
					NS = LA_STATE_IDLE;
				end
			LA_STATE_ARMED: begin			//trigger sequencer ARMED state (ready to trigger)
				if(TC_out)			//trigger?
					NS = LA_STATE_TRIGGERED;
				else
					NS = LA_STATE_ARMED;
				end
			LA_STATE_TRIGGERED: begin			//trigger sequencer TRIGGERED state (recording data)
				if(la_wd_p && (WriteAddress == 10'b1111111111))
					NS = LA_STATE_DONE;
				else
					NS = LA_STATE_TRIGGERED;
				end
			LA_STATE_DONE:	begin		//trigger sequencer DONE state (buffer is full)
				NS = LA_STATE_IDLE;
				end
		endcase
	end
	
	always @(la_wd_p or NS)
		if(la_wd_p && (NS == LA_STATE_TRIGGERED))
			la_wd_triggered = 1;
		else
			la_wd_triggered = 0;
	
/*
input clka;
input [15 : 0] dina;
input [9 : 0] addra;
input [0 : 0] wea;
output [15 : 0] douta;
input clkb;
input [7 : 0] dinb;
input [10 : 0] addrb;
input [0 : 0] web;
output [7 : 0] doutb;
*/

dpram16x8x2048 Ram1(
	.clka(clk),
	.dina({la_in_p[23:16],la_in_p[7:0]}),
	.addra(WriteAddress),
	.wea(la_wd_triggered),
	.douta(),
	.clkb(clk),
	.dinb(din),
	.addrb(ReadAddress[11:1]),
	.web(wdRam1),
	.doutb(dRam1)
);

dpram16x8x2048 Ram2(
	.clka(clk),
	.dina({la_in_p[31:24],la_in_p[15:8]}),
	.addra(WriteAddress),
	.wea(la_wd_p),
	.douta(),
	.clkb(clk),
	.dinb(din),
	.addrb(ReadAddress[11:1]),
	.web(wdRam2),
	.doutb(dRam2)
);



endmodule
