`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    14:36:37 09/14/2008 
// Design Name: 
// Module Name:    EnvelopeGen 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module EnvelopeGen(clk, reset, O, C, E, din, dout, wd, rd, wdCsr, rdCsr, A,LevelMux,TcMux,CS);
    input clk;				//system clock
    input reset;			//system reset
    output [15:0] O;		//signal output
    input [8:0] C;		//channel select
    input E;				//synth enable
    input [7:0] din;		//data from AVR
    output [7:0] dout;	//data to AVR	
    input wd;				//write data strobe
	 input rd;				//read ADRS registers
	 input wdCsr;			//write to control status memory
	 input rdCsr;			//read from control status memory
    input [11:0] A;		//address bus
	output [15:0] LevelMux,TcMux;
	output [2:0] CS;
	
	reg [7:0]dout;
	wire [15:0] O;

	reg [15:0] LevelMux;	//level mux
	reg [15:0] TcMux;		//time constant mux
	reg [2:0] CS;			//Current State
	wire [2:0] CSdat;		//current state right from RAM
	reg [2:0] NS;
	wire TrigDat;
	reg Trig;
	wire [15:0] EvO;
	wire [2:0] CSRdat;
	wire CSRtrig;

	reg Ed;				//delayed enable
	reg [8:0] Cd;		//delayed channel select
	
	wire [15:0] Attack,Decay,Release;	//time constants
	wire [15:0] Sustain;						//sustain level
	
	parameter STATE_RELEASE	=	3'b001;	//release state
	parameter STATE_ATTACK	=	3'b010;	//attack state
	parameter STATE_DECAY	=	3'b100;	//Decay State

	assign O = EvO;
	
	//-------------------------------------------------
	// Avr Interface Variables
	//-------------------------------------------------

	reg wdAD,wdSR,rdAD,rdSR;
	wire [7:0] ADdat,SRdat;
	
	always @(A or wd)
		if((A[11] == 0) && wd)
			wdAD = 1;
		else
			wdAD = 0;
			
	always @(A or wd)
		if((A[11] == 1) && wd)
			wdSR = 1;
		else
			wdSR = 0;

	always @(A or rd) 
		if((A[11] == 0) && rd)
			rdAD = 1;
		else
			rdAD = 0;
			
	always @(A or rd)
		if((A[11] == 1) && rd)
			rdSR = 1;
		else
			rdSR = 0;
			
	always @ (rdSR or SRdat or rdAD or ADdat or rdCsr or CSRdat or CSRtrig)
		if(rdSR) dout = SRdat;
		else if (rdAD) dout = ADdat;
		else if( rdCsr) dout = {CSRdat,4'b0,CSRtrig};
		else dout = 0;
		
	always @ (posedge clk)
		Ed <= E;
		
	always @ (posedge clk)
		Cd <= C;
		
/*-----------------------
input clka;
input [7 : 0] dina;
input [10 : 0] addra;
input [0 : 0] wea;
output [7 : 0] douta;
input clkb;
input [31 : 0] dinb;
input [8 : 0] addrb;
input [0 : 0] web;
output [31 : 0] doutb;
------------------------*/

dpram512x32x8 AttackDecay(
	.clka(clk),
	.dina(din),
	.addra(A[10:0]),
	.wea(wdAD),
	.douta(ADdat),
	.clkb(clk),
	.dinb(0),
	.addrb(C),
	.web(1'b0),
	.doutb({Attack,Decay})
);

/*-----------------------
input clka;
input [7 : 0] dina;
input [10 : 0] addra;
input [0 : 0] wea;
output [7 : 0] douta;
input clkb;
input [31 : 0] dinb;
input [8 : 0] addrb;
input [0 : 0] web;
output [31 : 0] doutb;
------------------------*/

dpram512x32x8 SustainRelease(
	.clka(clk),
	.dina(din),
	.addra(A[10:0]),
	.wea(wdSR),
	.douta(SRdat),
	.clkb(clk),
	.dinb(0),
	.addrb(C),
	.web(1'b0),
	.doutb({Release,Sustain})
);

onepolefilter EvTc (
    .clk(clk), 
    .reset(reset), 
    .I(LevelMux), 
    .O(EvO), 
    .F(TcMux), 
    .C(C), 
    .E(E)
    );

/*
input [8 : 0] a;
input [1 : 0] d;
input [8 : 0] dpra;
input clk;
input we;
output [1 : 0] spo;
output [1 : 0] dpo;
*/

dpRam512x3 State(
	.a(Cd),
	.d(NS),
	.dpra(C),
	.clk(clk),
	.we(Ed),
	.spo(),
	.dpo(CSdat)
);

always @ (posedge clk) CS <= CSdat;

dpRam512x3 StateStat(
	.a(Cd),
	.d(NS),
	.dpra(A[8:0]),
	.clk(clk),
	.we(Ed),
	.spo(),
	.dpo(CSRdat)
);

/*
input [8 : 0] a;
input [0 : 0] d;
input [8 : 0] dpra;
input clk;
input we;
output [0 : 0] spo;
output [0 : 0] dpo;
*/

dpram512x1 Trigger(
	.a(A[8:0]),
	.d(din[0]),
	.dpra(C),
	.clk(clk),
	.we(wdCsr),
	.spo(CSRtrig),
	.dpo(TrigDat)
);

always @(posedge clk) Trig <= TrigDat;

//---------------------------------
// State machine
//---------------------------------

always @ (CS or Trig or EvO) begin
 case (CS)
	STATE_RELEASE: begin
		if(Trig) begin
			NS = STATE_ATTACK;
		end
		else begin
			NS = STATE_RELEASE;
		end
	end
	STATE_ATTACK: begin
		if(EvO > 16'hff00) begin
			NS = STATE_DECAY;
		end
		else begin
			NS = STATE_ATTACK;
		end
	end
	STATE_DECAY: begin
		if(Trig) begin
			NS = STATE_DECAY;
		end
		else begin
			NS = STATE_RELEASE;
		end
	end
	default: begin
		NS = STATE_RELEASE;
	end
 endcase
end

//-----------------------------------
// time constant Mux
//-----------------------------------

always @ (CS or Attack or Decay or Release) begin
	case(CS)
		STATE_RELEASE: TcMux = Release;
		STATE_ATTACK: TcMux = Attack;
		STATE_DECAY:  TcMux = Decay;
		default: TcMux = Release;
	endcase
end

//-----------------------------------
// Level Mux
//-----------------------------------

always @(CS or Sustain) begin
	case(CS)
		STATE_RELEASE: LevelMux = 0;
		STATE_ATTACK: LevelMux = 16'hffff;
		STATE_DECAY: LevelMux = Sustain;
		default: LevelMux = Sustain;
	endcase
end


endmodule
