`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    16:50:40 09/20/2008 
// Design Name: 
// Module Name:    I2S 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
// I2S interface
// For interfacing up to I2S devices such as delta sigma converters
// Created by Jim Patchell for his FPGA based digital synthesizer
// Copyright (c) 2008 by James Patchell
// This code may be freely used as long as this notice remains in the file.
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module I2S(clk, reset, C, mck, sck, lrck, sd, I);
    input clk;					//system clock
    input reset;				//system reset
	 input [8:0] C;			//channel input
    output mck;				//master I2S clock
    output sck;				//I2S Serial Clock
    output lrck;				//left-right clock
    output sd;					//serial data
    input [15:0] I;			//data input
/****************************************************************
* for the time being, this is going to be mono-we send the same
* data out for both left and write channels
****************************************************************/

	wire mck;
	wire sck;
	wire lrck;
	wire sd;
	reg SS;
	
	reg [15:0] shifter;
	reg fd1,fd2;
	reg [15:0] DataHold;
	
	parameter ADR_MCLKDIV			=	2'b00;
	parameter ADR_LRDIV				=	2'b01;
	parameter ADR_SCKDIV				=	2'b10;
	
	//-----------------------------------------------------
	// Master Clock Divider
	//-----------------------------------------------------
	
	assign mck = C[1];
	assign sck = C[2];
	
	//----------------------------------------------------------
	// LR clock divider
	//----------------------------------------------------------
	
	assign lrck = C[8];
	
	//------------------------------------------------------
	// Shift register
	//------------------------------------------------------
	
	always @ (posedge clk or posedge reset)
		if(reset)
			DataHold <= 0;
		else if (C == 9'b111111110)
			DataHold <= I;
			
	always @ (posedge clk or posedge reset)
		if(reset)
			shifter <= 0;
		else if ((C == 9'b000000111) || (C == 9'b100000111))
			shifter <= DataHold;
		else if ((C[2:0] == 3'b111 ) && sck)
			shifter <= {shifter[14:0],1'b0};

		assign sd = shifter[15];
			
			
		
endmodule					
