`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    13:39:48 08/30/2008 
// Design Name: 
// Module Name:    SynthSequencer 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: Module to generate siginals to sequence synth modules throught
//				their execution cycle.  There are 512 channels, @ 50MHz, we can
//				get 96KHz sample rate (darn!)
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module SynthSequencer(clk, reset, E, C, SE);
    input clk;			//system clock
    input reset;		//system reset
    input E;			//enable sequencer
    output [8:0] C;	//channel address output
    output SE;			//Synth enable output
	
	wire [8:0] C;		//channel address register
	reg [8:0] CC;

	reg SE;
	
	reg [1:0] NS,CS;			//state machine variables

	parameter STATE_IDLE		=	2'b00;
	parameter STATE_WAIT		=	2'b01;
	parameter STATE_ACTIVE	=	2'b11;
	parameter STATE_DONE		= 	2'b10;
	
	parameter CTL_NOP							=	4'b0000;
	parameter CTL_SYHTH_EN					=	4'b0100;

	
	always @ (posedge clk or posedge reset)
		if(reset)
			CS <= STATE_IDLE;
		else
			CS <= NS;
	
	always @ (posedge clk or posedge reset)
		if(reset)
			CC <= 0;
		else
			CC <= CC + 1;

	assign C = CC;
	
	always @ (CS or E or CC) begin
		case (CS)
			STATE_IDLE: begin
				//------------------------------
				// wait for synth enable signal
				//------------------------------
				if(E) begin
					NS = STATE_WAIT;
					SE = 0;
				end
				else begin
					NS = STATE_IDLE;
					SE = 0;
				end
			end
			STATE_WAIT: begin
				//------------------------------
				// wait for sample signal
				//------------------------------
				if(CC == 9'b0) begin
					NS = STATE_ACTIVE;
					SE = 1;
				end
				else begin
					NS = STATE_WAIT;
					SE = 0;
				end
			end
			STATE_ACTIVE: begin
				//-------------------------------
				// process until all 512 channels
				// have been done
				//-------------------------------
				if((CC == 511) && (E == 0)) begin
					NS = STATE_DONE;
					SE = 1;
				end
				else begin
					NS = STATE_ACTIVE;
					SE = 1;
				end
			end
			STATE_DONE: begin
				//-------------------------------
				// if still enabled, wait for
				// sample signal to go true
				// otherwise, go to idle
				//-------------------------------
				SE = 0;
				NS = STATE_IDLE;
			end
		endcase
	end
	
endmodule
