`timescale 1ns / 1ps
////////////////////////////////////////////////////////////////////////////////
// Company: Zaca Design
// Engineer: Jim Patchell
//
// Create Date:    8:13:34 01/22/06
// Design Name:	AVR host interface to Spartan III 
// Module Name:    avrif
// Project Name: 
// Target Device:  
// Tool versions:  
// Description:
//
// Dependencies:
//
// Version 0.02 March 31, 2008
//		Added external memory interface
//		Changed interface to VGA display.  Made LUT and CTL regs seperate from
//			font memory and display memory
// 
////////////////////////////////////////////////////////////////////////////////
module avrif(clk, reset, ad, d, a, ale, wd, rd,oen,la,led,EA,irq,tirq,H,V,R,G,B,rDI,rDO,rAO,rCE1,rLB1,rUB1,rCE2,rUB2,rLB2,rWE,rOE,rboe,rxd,txd,uartIrq,mck, sck, lrck, sd,MidiIn,MidiOut);
   input clk;				//system clock
   output reset; 			//system reset
   input [7:0] ad;		//address data input
   output [7:0] d;		//data output
   input [7:0] a;			//upper address bits
   input ale;				//address latch eneable
   input wd;				//write data (active low)
   input rd;				//read data (active low)
	output oen;				//output tristate enable
	output [3:0]la,EA;
	output [7:0] led;
	output irq;
	output tirq;
	output H,V,R,G,B;
	
    input [31:0] rDI;		//RAM interface DATA in
    output [31:0] rDO;		//RAM interface DATA out
    output [17:0] rAO;		//Ram Interface Address OUT
    output rCE1;				//ram interface CE1
    output rUB1;				//ram interface Upper Byte 1
    output rLB1;				//ram interface Lower Byte 1
    output rCE2;				//ram interface CE2
    output rUB2;				//ram interface Upper Byte 2
    output rLB2;				//ram interface Lower Byte 2
    output rWE;				//ram interface Write Enable
    output rOE;					//ram Interface Read Enable
	 output rboe;			//tristate enable for sending data to ram
	 input rxd;
	 output txd;
	 output uartIrq;
	 output mck, sck, lrck, sd;
	 input MidiIn;
	 output MidiOut;
	 
	 wire mck, sck, lrck, sd;
	 wire uartIrq;
	 reg rboe;
    wire rCE1;				//ram interface CE1
    wire rUB1;				//ram interface Upper Byte 1
    wire rLB1;				//ram interface Lower Byte 1
    wire rCE2;				//ram interface CE2
    wire rUB2;				//ram interface Upper Byte 2
    wire rLB2;				//ram interface Lower Byte 2
    wire [31:0] rDO;		//RAM interface DATA out
    wire [17:0] rAO;		//Ram Interface Address OUT
    wire rWE;				//ram interface Write Enable
    wire rOE;					//ram Interface Read Enable


	
	reg reset;
	wire tirq;
	wire irq;
	wire [7:0] led;
	reg [3:0]la,EA;
	reg [7:0] LedOut;
	reg wdLed,rdLed;
	wire H,V,R,G,B;
	
	reg [7:0] d;
	reg [7:0] alow;		//lower portion of the address
	reg oen;
	reg wdedge, wdstrobe;

	wire [15:0] adr;
	
	reg wdCntr;
	reg rdStatus;
	
	//-------------------------------------------------------------------
	// address decoder stuff
	//-------------------------------------------------------------------

	//-------------------------------------------------------------------
	// Uart Interface
	//-------------------------------------------------------------------
	
	reg rdUart,wdUart,rdUartD,rdUartStb;
	wire [7:0] UartD;
	
	//-------------------------------------------------------------------
	// Midi Interface
	//-------------------------------------------------------------------
	
	reg rdMidi,wdMidi,rdMidiD,rdMidiStb;
	wire [7:0] MidiD;
	wire midiIRQ;
	//-------------------------------------------------------------------
	// VGA interface
	//-------------------------------------------------------------------
	
	wire [7:0] vgadata;
	reg wdVGA;
	reg rdVGA;
	reg wdLut,rdLut;		//color lookup table read and write
	wire [7:0] LutDat,CtlDat;
	reg wdVgaCTL,rdVgaCTL;	//vga control status reg read and write
	
	//-------------------------------------------------------------------
	// Control Port
	// Bit defines:
	//	CTL0:Enable Tone Generator
	//-------------------------------------------------------------------
	
	reg [7:0] CtlPort;
	reg wdCtl,rdCtl;
	
	//-------------------------------------------------------------------
	// timer
	//-------------------------------------------------------------------
	
	reg rdTimer,wdTimer;
	wire [7:0] TimerD;
	wire Timerirq;
	
	//-------------------------------------------------------------------
	// Ram Defines
	//-------------------------------------------------------------------
	
	reg wdRam,rdRam,rdRamM;
	wire [7:0] ramDat;
	
	//-------------------------------------------------------------------
	// Time of Day Defines
	//-------------------------------------------------------------------
	
	reg wdTod,rdTod;
	wire [7:0] TodDat;
	
	//-------------------------------------------------------------------
	// logic analyzer defines
	//-------------------------------------------------------------------
	
	reg wdLa,rdLa,rdLaStb,rdLaD;
	wire [7:0] LaDat;
	reg [15:0] LaQual;		//logic analyzer qualifier register
	reg wdLaQualL,wdLaQualH;
	reg rdLaQualL,rdLaQualH;
	reg WriteToLa;

	//-------------------------------------------------------------------
	// Synth Oscilator Defines
	//-------------------------------------------------------------------
	
	reg wdOSC,rdOSC;
	wire [15:0]sOSC;		//signal output from oscilator
	wire [7:0] OSCdat;	//data back to AVR
	wire [8:0] SynthChannel;	//current channel address
	wire SynthEnable;
	
	//-------------------------------------------------------------------
	// Synth Envelope Generator 1 defines
	//-------------------------------------------------------------------
	
	reg wdEV1,rdEV1,wdEv1CSR,rdEv1CSR;
	wire [15:0] sEV1;	//signal output from Envelope Generator 1
	wire [7:0] EV1dat;	//data back to AVR from Envelope Generator 1
	
	//-------------------------------------------------------------------
	// Synth Envelope Generator 2 Defines
	//-------------------------------------------------------------------

	reg wdEV2,rdEV2,wdEv2CSR,rdEv2CSR;
	wire [15:0] sEV2;	//signal output from Envelope Generator 2
	wire [7:0] EV2dat;	//data back to AVR from Envelope Generator 2
	
	//-------------------------------------------------------------------
	// vca
	//-------------------------------------------------------------------
	
	
	//-------------------------------------------------------------------
	// I2S interface defines
	//-------------------------------------------------------------------
	
	wire [15:0] sSynth;
	
	//-------------------------------------------------------------------
	// sequencer defines
	//-------------------------------------------------------------------
	wire SE;		//synthesizer enable signal
	wire cycDone;
	reg SE1,SE2,SE3,SE4;
	reg [8:0] SC1,SC2;
	//-------------------------------------------------------------------
	// test multiplier
	//-------------------------------------------------------------------
	
	reg [15:0] MultA,MultB;
	wire [33:0]MultP;
	reg [7:0] Mdat;
	reg wdMult;
	reg rdMult;
	
	//-------------------------------------------------------------------
	// math test
	//-------------------------------------------------------------------
	
	reg wdMT,rdMT;
	wire [7:0] MTout;
	
	//-------------------------------------------------------------------
	// Command Strobes
	//-------------------------------------------------------------------
	
	reg wdSingleStep;		//starts one sample cycle of synthesizer
	reg [8:0] SSChan;		//9 bit register for channel
	reg [7:0] SSChanDat;	//data back to avr
	reg wdSSChanL,wdSSChanH;			//write data to single step select reg
	reg rdSSChan;			//read data from single step select reg
	reg [15:0]SSosc,SSev,SSvca,SSLevelMux,SSTCMux;
	//----------------------------
	
	reg [7:0] sigData;		//signal data mux for reading synth registers
	reg rdSigData;
	wire [15:0]	 LevelMux,TcMux;
	wire [2:0] EVCS;
	
	//-------------------------------------------------------------------
	// Address defines
	//-------------------------------------------------------------------
	parameter ADR_RAM_START	  = 16'h2200;		//memory
	parameter ADR_RAM_END	  = 16'h7fff;

	parameter ADR_OSC_START		=	16'h8000;	//2048 byte block
	parameter ADR_OSC_END		=	16'h87ff;
	parameter ADR_FLT_START		=	16'h8800;	//2048 byte block
	parameter ADR_FLT_END		=	16'h8fff;
	parameter ADR_EV1_START		=	16'h9000;	//4096 byte block
	parameter ADR_EV1_END		=	16'h9fff;
	parameter ADR_EV2_START		=	16'ha000;	//4096 byte block
	parameter ADR_EV2_END		=	16'hafff;
	parameter AVR_EV1_CSR		=	16'hb000;	//512 byte block
	parameter AVR_EV1_CSR_END	=	16'hb1ff;
	parameter AVR_EV2_CSR		=	16'hb200;	//512 byte block
	parameter AVR_EV2_CSR_END	=	16'hb3ff;
	parameter ADR_VGA_START		=	16'hc000;	//vga
	parameter ADR_VGA_END		=	16'hefff;
	
	parameter STATUSLED 	= 16'hf000;	//led status port
	parameter ADR_RESET	= 16'hf001;	//reset port for fpga
	parameter ADR_VGA_CTL =	16'hf002;
	parameter ADR_CNTR =    16'hf003;
	parameter ADR_LA_QUAL_L		=	16'hf004;
	parameter ADR_LA_QUAL_H		=	16'hf005;
	parameter ADR_STATUS			=	16'hf006;
	parameter ADR_CMD_STROBES	=	16'hf006;
	parameter ADR_LUTRAM       =  16'hf010;
	parameter ADR_LUTRAM_END   =  16'hf01f;
	parameter ADR_CTL				=	16'hf020;	//control port
	parameter ADR_SS_CHANNEL_L	=	16'hf022;	//channel to record in SS
	parameter ADR_SS_CHANNEL_H	=	16'hf023;	//channel to record in SS
	parameter ADR_MULT			=	16'hf030;	//test multiplier
	parameter ADR_MULT_END		=	16'hf03f;
	parameter ADR_TIMER			=	16'hf040;	//timer port
	parameter ADR_UART_START	=	16'hf050;	//uart port
	parameter ADR_UART_END		=	16'hf057;
	parameter ADR_TOD				=	16'hf060;
	parameter ADR_TOD_END		=	16'hf06f;
	parameter ADR_DDS				=	16'hf070;	//DDS port
	parameter ADR_DDS_END		=	16'hf077;
	parameter ADR_LA				=	16'hf080;	//logic analyzer port
	parameter ADR_LA_END			=	16'hf08f;
	parameter ADR_OSC_L			=	16'hf090;	//oscilator signal
	parameter ADR_OSC_H			=	16'hf091;
	parameter ADR_EV1_L			=	16'hf092;	//ev1 signal
	parameter ADR_EV1_H			=	16'hf093;
	parameter ADR_EV2_L			=	16'hf094;	//ev2 signal
	parameter ADR_EV2_H			=	16'hf095;
	parameter ADR_FILT_L			=	16'hf096;	//filter signal
	parameter ADR_FILT_H			=	16'hf097;
	parameter ADR_VCA_L			=	16'hf098;	//vca signal
	parameter ADR_VCA_H			=	16'hf099;
	parameter ADR_CHAN_L			=	16'hf09a;	//channel select
	parameter ADR_CHAN_H			=	16'hf09b;
	parameter ADR_MATHTEST		=	16'hf0a0;	//math test
	parameter ADR_MATHTEST_END	=	16'hf0af;	
	parameter ADR_MIDI			=	16'hf0b0;	//midi interface
	parameter ADR_MIDI_END		=	16'hf0b7;
	
	//-----------------------------------------------------------------
	// test points
	//-----------------------------------------------------------------
/*	
	assign tp[0] = wdstrobe;
	assign tp[1] = wdLed;
	assign tp[2] = oen;
	assign tp[3] = rdLed;
*/
	//--------------------------------------------------
	// address latch
	//--------------------------------------------------

	always @ (negedge ale) begin
		alow <= ad;			//latch address on falling edge of ale
	end

	assign adr = {a,alow};	//generate internal address bus

	//--------------------------------------------------
	// generate write data strobe
	//--------------------------------------------------

	always @ (negedge wd or posedge wdstrobe) begin
		if(wdstrobe)
			wdedge <= 0;	//reset
		else
			wdedge <= 1;	//set
	end

	always @ (posedge clk)
		wdstrobe <= wdedge;

	//-----------------------------------------------------
	// Control Register
	//-----------------------------------------------------
	
	always @ (posedge clk)
		if(wdCtl) CtlPort <= ad;

	assign SynthEnable = CtlPort[0];
	
	//-----------------------------------------------------
	// Command Write STrobes
	//-----------------------------------------------------
	
	always @ (adr or wdstrobe or ad)
		if((adr == ADR_CMD_STROBES) && wdstrobe && ad[0])
			wdSingleStep = 1;
		else
			wdSingleStep = 0;
	//-----------------------------------------------------
	// Address decode Write Logic
	//-----------------------------------------------------
	
	always @ (adr or wdstrobe)
		if((adr == ADR_SS_CHANNEL_L) && wdstrobe)
			wdSSChanL = 1;
		else
			wdSSChanL = 0;

	always @ (adr or wdstrobe)
		if((adr == ADR_SS_CHANNEL_H) && wdstrobe)
			wdSSChanH = 1;
		else
			wdSSChanH = 0;
			
	always @ (adr or wdstrobe)
		if((adr >= ADR_MATHTEST) && (adr <= ADR_MATHTEST_END) && wdstrobe)
			wdMT = 1;
		else
			wdMT = 0;
			
	always @ (adr or wdstrobe)
		if((adr >= ADR_MULT) && (adr <= ADR_MULT_END) && wdstrobe)
			wdMult = 1;
		else
			wdMult = 0;
			
	always @ (adr or wdstrobe)
		if((adr >= AVR_EV1_CSR) && (adr <= AVR_EV1_CSR_END) && wdstrobe)
			wdEv1CSR = 1;
		else
			wdEv1CSR = 0;

	always @ (adr or wdstrobe)
		if((adr >= AVR_EV2_CSR) && (adr <= AVR_EV2_CSR_END) && wdstrobe)
			wdEv2CSR = 1;
		else
			wdEv2CSR = 0;
			
	always @ (adr or wdstrobe)
		if((adr >= ADR_EV1_START) && (adr <= ADR_EV1_END) && wdstrobe)
			wdEV1 = 1;
		else
			wdEV1 = 0;
			
	always @ (adr or wdstrobe)
		if((adr >= ADR_EV2_START) && (adr <= ADR_EV2_END) && wdstrobe)
			wdEV2 = 1;
		else
			wdEV2 = 0;
	
	always @ (adr or wdstrobe)
		if((adr == ADR_LA_QUAL_L) && wdstrobe)
			wdLaQualL = 1;
		else
			wdLaQualL = 0;
			
	always @ (adr or wdstrobe)
		if((adr == ADR_LA_QUAL_H) && wdstrobe)
			wdLaQualH = 1;
		else
			wdLaQualH = 0;
			
	always @ (adr or wdstrobe)
		if((adr >= ADR_OSC_START) && (adr <= ADR_OSC_END) && wdstrobe)
			wdOSC = 1;
		else
			wdOSC = 0;
	
	always @ (adr or wdstrobe or ad)
		if((adr == ADR_CNTR) && (wdstrobe) && (ad == 8'h01))
			wdCntr = 1;
		else
			wdCntr = 0;
	

	always @ (adr or wdstrobe or ad)
		if((adr == ADR_RESET) && (wdstrobe) && (ad == 8'h5a))
			reset = 1;
		else
			reset = 0;
			
	always @ (adr or wdstrobe) begin
		if((adr == STATUSLED) && (wdstrobe))
			wdLed = 1;
		else
			wdLed = 0;
	end

	always @ (adr or wdstrobe) begin
		if((adr == ADR_CTL) && wdstrobe)
			wdCtl = 1;
		else
			wdCtl = 0;
	end
		
	always @ (adr or wdstrobe)
		if( (adr >= ADR_TIMER) && (adr <= ADR_TIMER + 7) && wdstrobe)
			wdTimer = 1;
		else
			wdTimer = 0;
	
	always @ (adr or wdstrobe)
		if( (adr >= ADR_VGA_START) && (adr <= ADR_VGA_END) && wdstrobe)
			wdVGA = 1;
		else
			wdVGA = 0;
			
	always @ (adr or rd)
		if( (adr >= ADR_VGA_START) && (adr <= ADR_VGA_END) && (rd == 0))
			rdVGA = 1;
		else
			rdVGA = 0;

	always @ (adr or wd)
		if ( (adr >= ADR_RAM_START) && (adr <= ADR_RAM_END) && (wd==0))
			wdRam = 1;
		else
			wdRam = 0;
			
	always @ (adr or rd)
		if ( (adr >= ADR_RAM_START) && (adr <= ADR_RAM_END) && (rd == 0))
			rdRam = 1;
		else
			rdRam = 0;

	always @ (adr or wd)
		if ( (adr >= ADR_RAM_START) && (adr <= ADR_RAM_END) && (wd == 0))
			rboe = 0;	//active low signal
		else
			rboe = 1;
	
	always @ (adr or wdstrobe)
		if( (adr >= ADR_UART_START) && (adr <= ADR_UART_END) && wdstrobe)
			wdUart = 1;
		else
			wdUart = 0;
			
	always @ (negedge rd or posedge rdMidi)
		if(rdMidi)
			rdMidiStb <= 0;
		else if (adr == ADR_MIDI)
			rdMidiStb <= 1;
			
	always @ (posedge clk)
		rdMidi <= rdMidiStb;
		
	always @ (adr or wdstrobe)
		if(( adr >= ADR_MIDI) && (adr <= ADR_MIDI_END) && wdstrobe)
			wdMidi = 1;
		else
			wdMidi = 0;
			
	always @ (negedge rd or posedge rdUart)
		if(rdUart)
			rdUartStb = 0;
		else if( (adr >= ADR_UART_START) && (adr <= ADR_UART_END) )
			rdUartStb = 1;
			
	always @(posedge clk)
		rdUart = rdUartStb;
		
	always @ (adr or wdstrobe)
		if(( adr >= ADR_TOD) && (adr <= ADR_TOD_END) && wdstrobe)
			wdTod = 1;
		else
			wdTod = 0;
			
	always @ (adr or rd)
		if(( adr >= ADR_TOD) && (adr <= ADR_TOD_END) && (rd == 0))
			rdTod = 1;
		else
			rdTod = 0;

	always @ (adr or wdstrobe)
		if(( adr >= ADR_LA) && (adr <= ADR_LA_END) && wdstrobe)
			wdLa = 1;
		else
			wdLa = 0;
			
	always @ (posedge rd or posedge rdLa)	//trigger at end of read
		if(rdLa)
			rdLaStb <= 0;
		else if ((adr >= ADR_LA) && (adr <= ADR_LA_END))
			rdLaStb <= 1;
			
	always @ (posedge clk)
		rdLa <= rdLaStb;

	always @ (adr)
		if((adr >= ADR_OSC_START) && (adr <= ADR_OSC_END))
			rdOSC = 1;
		else
			rdOSC = 0;

	always @ (adr)
		if(adr == ADR_LA_QUAL_L)
			rdLaQualL = 1;
		else
			rdLaQualL = 0;

	always @ (adr)
		if(adr == ADR_LA_QUAL_H)
			rdLaQualH = 1;
		else
			rdLaQualH = 0;

	//-----------------------------------------------------------------------------
	// Output enable for data bus tri state
	//-----------------------------------------------------------------------------
	always @ (adr or rd or rdVGA or rdRamM or rdOSC or rdEv1CSR or rdEv2CSR or rdEV2 or rdEV1) begin
	//------------------------------------------
	// rd is active low
	//------------------------------------------
		if(rd == 0) begin
			if(adr[15:8] == 8'hf0) oen = 0;
			else if (rdVGA) oen = 0;
			else if (rdRamM) oen = 0;
			else if (rdOSC) oen = 0;
			else if (rdEV1) oen = 0;
			else if (rdEV2) oen = 0;
			else if (rdEv1CSR) oen = 0;
			else if (rdEv2CSR) oen = 0;
			else oen = 1;
		end
		else
			oen = 1;
	end

	//---------------------------------------------------
	// mux for reading stuff back
	//---------------------------------------------------

	always @ (adr)
		if(adr == STATUSLED)
			rdLed = 1;
		else
			rdLed = 0;

	always @ (adr)
		if(adr == ADR_CTL)
			rdCtl = 1;
		else
			rdCtl = 0;

	always @ (adr)
		if( (adr >= ADR_TIMER) && (adr <= ADR_TIMER + 7))
			rdTimer = 1;
		else
			rdTimer = 0;

	always @ (adr)
		if( (adr >= ADR_LUTRAM) && (adr <= ADR_LUTRAM_END))
			rdLut = 1;
		else
			rdLut = 0;

	always @ (adr)
		if( (adr == ADR_VGA_CTL) )
			rdVgaCTL = 1;
		else
			rdVgaCTL = 0;

	always @ (adr)
		if ( (adr >= ADR_RAM_START) && (adr <= ADR_RAM_END))
			rdRamM = 1;
		else
			rdRamM = 0;
			
	always @ (adr)
		if ( (adr >= ADR_UART_START) && (adr <= ADR_UART_END) )
			rdUartD = 1;
		else
			rdUartD = 0;

	always @ (adr)
		if((adr >= ADR_LA) && (adr <= ADR_LA_END))
			rdLaD = 1;
		else
			rdLaD = 0;

	always @ (adr)
		if((adr >= ADR_EV1_START) && (adr <= ADR_EV1_END))
			rdEV1 = 1;
		else
			rdEV1 = 0;

	always @ (adr)
		if((adr >= ADR_EV2_START) && (adr <= ADR_EV2_END))
			rdEV2 = 1;
		else
			rdEV2 = 0;

	always @ (adr)
		if((adr >= AVR_EV1_CSR) && (adr <= AVR_EV1_CSR_END))
			rdEv1CSR = 1;
		else
			rdEv1CSR = 0;

	always @ (adr)
		if((adr >= AVR_EV2_CSR) && (adr <= AVR_EV2_CSR_END))
			rdEv2CSR = 1;
		else
			rdEv2CSR = 0;

	always @ (adr)
		if(adr == ADR_STATUS)
			rdStatus = 1;
		else
			rdStatus = 0;

	always @ (adr)
		if((adr >= ADR_MULT) && (adr <= ADR_MULT_END))
			rdMult = 1;
		else
			rdMult = 0;

	always @ (adr)
		if((adr >= ADR_OSC_L) && (adr <= ADR_CHAN_H))
			rdSigData = 1;
		else
			rdSigData = 0;

	always @ (adr)
		if((adr >= ADR_MATHTEST) && (adr <= ADR_MATHTEST_END))
			rdMT = 1;
		else
			rdMT = 0;

	always @ (adr)
		if(( adr >= ADR_MIDI) && (adr <= ADR_MIDI_END))
			rdMidiD = 1;
		else
			rdMidiD = 0;

	always @ (adr)
		if((adr == ADR_SS_CHANNEL_L) || (adr == ADR_SS_CHANNEL_H))
			rdSSChan = 1;
		else
			rdSSChan = 0;
			
	always @ (rdLed or LedOut or 
				rdCtl or CtlPort or rdVGA or vgadata or
				rdTimer or TimerD or rdLut or LutDat or rdVgaCTL or CtlDat or rdRamM or ramDat
				or rdUartD or UartD or rdTod or TodDat or rdLaD or LaDat
				or rdEV1 /*or rdEV2*/ or EV1dat/* or EV2dat*/ or rdEv1CSR/* or rdEv2CSR */ or rdStatus or cycDone
				or rdOSC or OSCdat or rdLaQualL or LaQual or rdLaQualH or rdMult or rdSigData or sigData
				or rdMT or MTout or rdMidiD or MidiD) begin
		if(rdLed) d = LedOut;
		else if (rdCtl) d = CtlPort;
		else if (rdRamM) d = ramDat;
		else if (rdTimer) d = TimerD;
		else if (rdVGA) d = vgadata;
		else if (rdLut) d = LutDat;
		else if (rdVgaCTL) d = CtlDat;
		else if (rdUartD) d = UartD;
		else if (rdTod) d = TodDat;
		else if (rdLaD) d = LaDat;
		else if (rdOSC) d = OSCdat;
		else if (rdLaQualL) d = LaQual[7:0];
		else if (rdLaQualH) d = LaQual[15:8];
		else if (rdEV1 || rdEv1CSR) d = EV1dat;
//		else if (rdEV2) d = EV2dat;
//		else if (rdEv2CSR) d = EV2dat;
		else if (rdStatus) d = {7'b0,cycDone};
		else if (rdMult) d = Mdat;
		else if (rdSigData) d = sigData;
		else if (rdMT) d = MTout;
		else if (rdMidiD) d = MidiD;
		else if (rdSSChan) d = SSChanDat;
		else
			d = 8'b01111011;
	end

	//--------------------------------------------------
	// Test Multiplier
	//--------------------------------------------------
	
	always @ (posedge clk or posedge reset) begin
		if(reset)
			MultA <= 0;
		else begin
			if((adr[2:0] == 3'b000) && wdMult)
				MultA[7:0] <= ad;
			else
				MultA[7:0] <= MultA[7:0];
			if ((adr[2:0] == 3'b001) && wdMult)
				MultA[15:8] <= ad;
			else
				MultA[15:8] <= MultA[15:8];
		end
	end
	
	always @ (posedge clk or posedge reset) begin
		if(reset) begin
			MultB <= 0;
		end
		else begin
			if(wdMult) begin
				if ((adr[2:0] == 3'b010) && wdMult)
					MultB[7:0] <= ad;
				else
					MultB[7:0] <= MultB[7:0];
				if ((adr[2:0] == 3'b011) && wdMult)
					MultB[15:8] <= ad;
				else
					MultB[15:8] <= MultB[15:8];
			end
		end
	end
	
	always @ (adr or MultA or MultB or MultP)
		case (adr[3:0])
			4'b0000: Mdat = MultA[7:0];
			4'b0001: Mdat = MultA[15:8];
			4'b0010: Mdat = MultB[7:0];
			4'b0011: Mdat = MultB[15:8];
			4'b0100: Mdat = MultP[7:0];
			4'b0101: Mdat = MultP[15:8];
			4'b0110: Mdat = MultP[23:16];
			4'b0111: Mdat = MultP[31:24];
			4'b1000: Mdat = {6'b0,MultP[33:32]};
			default:Mdat = 0;
		endcase


 
 
// The following must be inserted into your Verilog file for this
// core to be instantiated. Change the instance name and port connections
// (in parentheses) to your own signal names.
 
MultS17x17 YourInstanceName (
    .clk(clk),
    .a({MultA[15],MultA}), // Bus [16 : 0] 
    .b({1'b0,MultB}), // Bus [16 : 0] 
    .p(MultP)); // Bus [33 : 0] 

 
		
		
/*
  input clk;
  input [15 : 0] a;	//signed input
  input [15 : 0] b;	//unsigned input
  output [31 : 0] p;	//product
*/
/*
MultSAUB16 TestMult(
	.clk(clk),
	.a(MultA),
	.b(MultB),
	.p(MultP)
);
*/

MathTest MTest1 (
    .clk(clk), 
    .reset(reset), 
    .Din(ad), 
    .Dout(MTout), 
    .wd(wdMT), 
    .A(adr[3:0])
    );

	//--------------------------------------------------
	// Logic Analyzier Data qualifier Register
	//---------------------------------------------------
	
	always @ (posedge clk or posedge reset)
		if(reset)
			LaQual <= 0;
		else if (wdLaQualL)
			LaQual[7:0] <= ad;
		else if (wdLaQualH)
			LaQual[15:8] <= ad;
			
	//---------------------------------------------------
	// Data register for status LEDs
	//---------------------------------------------------

	always @ (posedge clk or posedge reset) begin
		if(reset)
			LedOut <= 8'b00000000;
		else begin
			if(wdLed) LedOut <= ad;
		end
	end

	assign led = LedOut;

	always @ (adr or wdstrobe)
		if((adr >= ADR_LUTRAM) && (adr <= ADR_LUTRAM_END) && wdstrobe)
			wdLut = 1;
		else
			wdLut = 0;

	always @ (adr or wdstrobe)
		if((adr == ADR_VGA_CTL) && wdstrobe)
			wdVgaCTL = 1;
		else
			wdVgaCTL = 0;
			
vga #(.VGA_BASE_ADDRESS(ADR_VGA_START)) vga1 (
    .clk(clk), 
    .reset(reset), 
    .wd(wdVGA), 
    .A(adr), 
    .D(ad), 
    .O(vgadata), 
    .H(H), 
    .V(V), 
    .R(R), 
    .G(G), 
    .B(B),
	 .vIrq(vgaIRQ),
	 .lO(LutDat),
	 .cO(CtlDat),
	 .wdLut(wdLut),
	 .wdCTL(wdVgaCTL)
);

//----------------------------------------------------
// external ram interface
//----------------------------------------------------
/*
    output [20:0] A,
    input [7:0] Di,
    output [7:0] Do,
    output W,
    output R,
    input [31:0] rDI,
    output [31:0] rDO,
    output [17:0] rAO,
    output rCE1,
    output rUB1,
    output rLB1,
    output rCE2,
    output rUB2,
    output rLB2,
    output rWE,
    output rOE

*/

ExtRam Ram1 (
    .A({4'b0,adr}), 
    .Di(ad), 
    .Do(ramDat), 
    .W(wdRam), 
    .R(rdRam), 
    .rDI(rDI), 
    .rDO(rDO), 
    .rAO(rAO), 
    .rCE1(rCE1), 
    .rUB1(rUB1), 
    .rLB1(rLB1), 
    .rCE2(rCE2), 
    .rUB2(rUB2), 
    .rLB2(rLB2), 
    .rWE(rWE), 
    .rOE(rOE)
    );

//----------------------------------------------------
// system timer
//----------------------------------------------------

timer timer1 (
    .clk(clk), 
    .reset(reset), 
    .a(adr[2:0]), 
	 .O(TimerD),
    .d(ad), 
    .wdstb(wdTimer), 
    .irq(Timerirq) 
    );

assign tirq = ~Timerirq;
assign irq = vgaIRQ;

//---------------------------------------------------
// UART
//---------------------------------------------------

muart Uart1 (
    .clk(clk), 
    .reset(reset), 
    .D(ad), 
    .O(UartD), 
    .A(adr[2:0]), 
    .wd(wdUart), 
    .rd(rdUart), 
    .I(uartIrq), 
    .rxd(rxd), 
    .txd(txd)
);

midiuart Midi (
    .clk(clk), 
    .reset(reset), 
    .D(ad), 
    .O(MidiD), 
    .A(adr[2:0]), 
    .wd(wdMidi), 
    .rd(rdMidi), 
    .I(midiIrq), 
    .rxd(MidiIn), 
    .txd(MidiOut)
    );

//---------------------------------------------------
// Time of Day
//---------------------------------------------------

tod TmOfDay (
    .clk(clk), 
    .reset(reset), 
    .I(ad), 
    .O(TodDat), 
    .A(adr[3:0]), 
    .W(wdTod)
    );

//---------------------------------------------------
// single step hardware
//---------------------------------------------------

//write data into single step select register

always @ (posedge clk) if(wdSSChanL) SSChan[7:0] <= ad;
always @ (posedge clk) if(wdSSChanH) SSChan[8] <= ad[0];

always @ (adr)
	if(adr[0])
		SSChanDat = {7'b0,SSChan[8]};
	else
		SSChanDat = SSChan[7:0];


always @ (posedge clk)
	if(SSChan == SC1)
		SSosc <= sOSC;

always @ (posedge clk)
	if(SSChan == SC1)
		SSev <= sEV1;

always @ (posedge clk)
	if(SSChan == SC1)
		SSLevelMux <= LevelMux;
		
always @ (posedge clk)
	if(SSChan == SC1)
		SSTCMux <= TcMux;
		
always @ (posedge clk)
	if((SC2 == 0) && SE3)
		SSvca <= sSynth;
		
//---------------------------------------------------
// Simple Logic Analyzer module
//---------------------------------------------------

always @ (SynthChannel or LaQual or SE)
	if((SynthChannel == LaQual[8:0]) && SE)
		WriteToLa = 1;
	else
		WriteToLa = 0;
		
Debug la1 (
    .clk(clk), 
    .reset(reset), 
    .la_in({SE,6'b0,SynthChannel,sSynth}), 
    .la_wd(WriteToLa), 
    .din(ad), 
    .dout(LaDat), 
    .rd(rdLa), 
    .wd(wdLa), 
    .A(adr[3:0])
);

//-----------------------------------------------------
// Synthesizer
//-----------------------------------------------------
	always @ (adr)
		case (adr[3:0])
			ADR_OSC_L[3:0]: sigData = SSosc[7:0];
			ADR_OSC_H[3:0]: sigData = SSosc[15:8];
			ADR_EV1_L[3:0]: sigData = SSev[7:0];
			ADR_EV1_H[3:0]: sigData = SSev[15:8];
			ADR_EV2_L[3:0]: sigData = SSTCMux[7:0];
			ADR_EV2_H[3:0]: sigData = SSTCMux[15:8];
			ADR_FILT_L[3:0]: sigData = SSLevelMux[7:0];
			ADR_FILT_H[3:0]: sigData = SSLevelMux[15:8];
			ADR_VCA_L[3:0]: sigData = SSvca[7:0];
			ADR_VCA_H[3:0]: sigData = SSvca[15:8];
			ADR_CHAN_L[3:0]: sigData = SynthChannel[7:0];
			ADR_CHAN_H[3:0]: sigData = {EVCS,4'b0,SynthChannel[8]};
			default:sigData = 0;
		endcase

assign sSE = wdSingleStep | SynthEnable;

always @ (posedge clk) SE1 <= SE;
always @ (posedge clk) SE2 <= SE1;
always @ (posedge clk) SE3 <= SE2;
always @ (posedge clk) SE4 <= SE3;

always @ (posedge clk) SC1 <= SynthChannel;
always @ (posedge clk) SC2 <= SC1;

SynthSequencer Seq (
    .clk(clk), 
    .reset(reset), 
    .E(sSE), 
    .C(SynthChannel), 
    .SE(SE)
   );

Osciclator Osc1 (
    .clk(clk), 
    .reset(reset), 
    .O(sOSC), 
    .C(SynthChannel), 
    .E(SE), 
    .din(ad), 
    .dout(OSCdat), 
    .wd(wdOSC), 
    .A(adr[10:0])
    );

EnvelopeGen EV1 (
    .clk(clk), 
    .reset(reset), 
    .O(sEV1), 
    .C(SynthChannel), 
    .E(SE), 
    .din(ad), 
    .dout(EV1dat), 
    .wd(wdEV1),
	 .rd(rdEV1),
    .wdCsr(wdEv1CSR),
	 .rdCsr(rdEv1CSR),
    .A(adr[11:0]),
	 .LevelMux(LevelMux),
	 .TcMux(TcMux),
	 .CS(EVCS)
    );

Mixer VcaMixer
 (
    .clk(clk), 
    .E(SE2),
	 .C(SC2),
    .I(sOSC), 
    .EV(sEV1), 
    .O(sSynth)
    );

I2S AnalogOut (
    .clk(clk), 
    .reset(reset),
	 .C(SC2),
    .mck(mck), 
    .sck(sck), 
    .lrck(lrck), 
    .sd(sd), 
    .I(sSynth)
    );

always @ (posedge clk)
 if (wdCntr)
	{EA,la} = {EA,la} + 1;

endmodule
