`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    17:31:34 07/13/2008 
// Design Name: 
// Module Name:    phaseAccumulator 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module phaseAccumulator(clk, reset, D, O, W, A, P,I);
    input clk;
    input reset;
    input [7:0] D;
    output [7:0] O;
    input W;
    input [2:0] A;
    output [31:0] P;
	 output I;

	reg [31:0] P;
	reg [31:0] Freq;
	reg [7:0] O;
	reg wdLo,wdMLo,wdMHi,wdHi,wdCtl,wdDiv,wdDivRL;
	reg [7:0] Div;
	reg [7:0] DivReload;
	reg incP;
	
	wire I;
	
	parameter PA_FREQ_LO		=		3'b000;
	parameter PA_FREQ_MLO	=		3'b001;
	parameter PA_FREQ_MHI	=		3'b010;
	parameter PA_FREQ_HI		=		3'b011;
	parameter PA_CTL			=		3'b100;
	parameter PA_DIV			=		3'b110;
	parameter PA_DIVRL		=		3'b111;
	
	always @ (W or A)
		if(W && (A == PA_FREQ_LO))
			wdLo = 1;
		else
			wdLo = 0;
			
	always @ (W or A)
		if(W && (A == PA_FREQ_MLO))
			wdMLo = 1;
		else
			wdMLo = 0;

	always @ (W or A)
		if(W && (A == PA_FREQ_MHI))
			wdMHi = 1;
		else
			wdMHi = 0;

	always @ (W or A)
		if(W && (A == PA_FREQ_HI))
			wdHi = 1;
		else
			wdHi = 0;

	always @ (W or A)
		if(W && (A == PA_CTL))
			wdCtl = 1;
		else
			wdCtl = 0;

	always @ (W or A)
		if(W && (A == PA_DIV))
			wdDiv = 1;
		else
			wdDiv = 0;

	always @ (W or A)
		if(W && (A == PA_DIVRL))
			wdDivRL = 1;
		else
			wdDivRL = 0;


	always @ (posedge clk or posedge reset)
		if(reset)
			Freq <= 0;
		else begin
			if(wdLo)
				Freq[7:0] <= D;
			else if (wdMLo)
				Freq[15:8] <= D;
			else if (wdMHi)
				Freq[23:16] <= D;
			else if (wdHi)
				Freq[31:24] <= D;
		end
		
		always @ (posedge clk or posedge reset)
			if(reset)
				P <= 0;
			else begin
				if(incP)
					P <= P + Freq;
			end
			
		always @ (posedge clk or posedge reset)
			if(reset)
				Div <= 0;
			else begin
				if(wdDiv)
					Div <= D;
				else if (Div == 0)
					Div <= DivReload;
				else
					Div <= Div - 1;
			end
			
		always @ (posedge clk or posedge reset)
			if(reset)
				DivReload <= 0;
			else begin
				if(wdDivRL)
					DivReload <= D;
			end
		
		always @(Div)
			if(Div == 0)
				incP = 1;
			else
				incP = 0;
				
		always @(A) begin
			case (A)
				PA_FREQ_LO: O = Freq[7:0];
				PA_FREQ_MLO: O = Freq[15:8];
				PA_FREQ_MHI: O = Freq[23:16];
				PA_FREQ_HI: O = Freq[31:24];
				PA_CTL: O = 8'h7c;
				PA_DIV: O = Div;
				PA_DIVRL: O = DivReload;
				default: O = 8'hb7;
			endcase
		end
		
endmodule
