`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    08:35:51 04/30/2008 
// Design Name: 
// Module Name:    tod 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module tod(
    input clk,
    input reset,
    input [7:0] I,
    output reg[7:0] O,
    input [3:0] A,
    input W
    );

	parameter TOD_ADR_SEC	= 0;
	parameter TOD_ADR_MIN	= 1;
	parameter TOD_ADR_HOUR	= 2;
	parameter TOD_ADR_DAY	= 3;
	parameter TOD_ADR_DOM	= 4;
	parameter TOD_ADR_MON	= 5;
	parameter TOD_ADR_YEAR	= 6;
	parameter TOD_ADR_LATCH		= 7;
	
	parameter SEC_ADR_LSB	= 8;
	parameter SEC_ADR_MLB	= 9;
	parameter SEC_ADR_MHB	= 10;
	parameter SEC_ADR_MSB	= 11;
	
	reg [7:0] SecR,MinR,HourR,DayR;
	wire [7:0] SecD,MinD,HourD,DayD;
	wire ES;
	reg EM,EH,ED;		//enable count
	reg LS,LM,LH,LDay,Latch;	//load register
	
	reg [31:0] SecHold,SecLatch;
	reg [15:0] SecondsL,SecondsH;
	reg wdSLsb,wdSMlb,wdSMhb,wdSMsb,ldSecs;
	
	wire Vs,Vm,Vh;
	
	reg [26:0] TimeBase;
	reg Tick;
	
	always @ (posedge clk or posedge reset)
		if(reset)
			TimeBase <= 0;
		else if(TimeBase == 0)
			TimeBase <= 49999999;
		else
			TimeBase <= TimeBase - 1;
			
	always @ (TimeBase)
		if(TimeBase == 0)
			Tick = 1;
		else
			Tick = 0;

	
	always @(A or W)
		if((A == TOD_ADR_SEC) && W)
			LS = 1;		//load
		else
			LS = 0;		//count
	
	assign ES = Tick;

	
	always @(Tick or Vs)
		if(Tick && Vs )
			EM = 1;
		else
			EM = 0;
			
	always @(Tick or Vs or Vm)
		if(Tick && Vs &&  Vm)
			EH = 1;
		else
			EH = 0;
						
	always @ (Tick or Vs or Vm or Vh)
		if(Tick && Vs  && Vm  && Vh )
			ED = 1;
		else
			ED = 0;
		
	always @ (A or W)
		if((A == TOD_ADR_MIN) && W)
			LM = 1;
		else
			LM = 0;

	always @(A or W)
		if((A == TOD_ADR_HOUR) && W)
			LH = 1;
		else
			LH = 0;
			
	always @(A or W)
		if((A == TOD_ADR_DAY) && W)
			LDay = 1;
		else
			LDay = 0;
			
	always @ (A or W)
		if((A == TOD_ADR_LATCH) && W)
			Latch = 1;
		else
			Latch = 0;

	always @ (A or W)
		if(( A == SEC_ADR_LSB) && W)
			wdSLsb = 1;
		else
			wdSLsb = 0;

	always @ (A or W)
		if(( A == SEC_ADR_MLB) && W)
			wdSMlb = 1;
		else
			wdSMlb = 0;

	always @ (A or W)
		if(( A == SEC_ADR_MHB) && W)
			wdSMhb = 1;
		else
			wdSMhb = 0;

	always @ (A or W)
		if(( A == SEC_ADR_MSB) && W)
			wdSMsb = 1;
		else
			wdSMsb = 0;
			
	always @ (posedge clk)
		if(Latch) begin
			SecR <= SecD;
			MinR <= MinD;
			HourR <= HourD;
			DayR <= DayD;
		end
	
	always @(SecR or MinR or HourR or DayR or A or SecLatch) begin
		case(A)
			TOD_ADR_SEC :O = SecR;
			TOD_ADR_MIN :O = MinR;
			TOD_ADR_HOUR:O = HourR;
			TOD_ADR_DAY :O = DayR;
			TOD_ADR_DOM :O = 8'h5a;
			TOD_ADR_YEAR:O = 8'h5a;
			SEC_ADR_LSB :O = SecLatch[7:0];
			SEC_ADR_MLB :O = SecLatch[15:8];
			SEC_ADR_MHB :O = SecLatch[23:16];
			SEC_ADR_MSB :O = SecLatch[31:24];
			default:O = 8'h5a;
		endcase
	end
	
	always @ (posedge clk)
		if(Tick)
			SecondsL <= SecondsL + 1;
		else if(ldSecs)
			SecondsL <= SecHold[15:0];
			
	always @ (posedge clk)
		if(Tick && (SecondsL == 16'hffff))
			SecondsH <= SecondsH + 1;
		else if(ldSecs)
			SecondsH <= SecHold[31:16];

	always @ (posedge clk)
		if(Latch)
			SecLatch <= {SecondsH,SecondsL};
			
	always @ (posedge clk)
		if(wdSLsb)
			SecHold[7:0] <= I;

	always @ (posedge clk)
		if(wdSMlb)
			SecHold[15:8] <= I;

	always @ (posedge clk)
		if(wdSMhb)
			SecHold[23:16] <= I;

	always @ (posedge clk)
		if(wdSMsb)
			SecHold[31:24] <= I;
			
	always @ (posedge clk)
		ldSecs <= wdSMsb;
	
bcdINC Sec (
    .clk(clk), 
    .reset(reset), 
    .I(I), 
    .O(SecD), 
    .E(LS), 
    .C(ES),
	 .V(Vs)
    );

bcdINC Min (
    .clk(clk), 
    .reset(reset), 
    .I(I), 
    .O(MinD), 
    .E(LM), 
    .C(EM),
	 .V(Vm)
    );

bcdINC #(.BCD_MAX(8'h23)) Hour (
    .clk(clk), 
    .reset(reset), 
    .I(I), 
    .O(HourD), 
    .E(LH), 
    .C(EH),
	 .V(Vh)
    );


bcdINC #(.BCD_MAX(8'h06) ) Day(
    .clk(clk), 
    .reset(reset), 
    .I(I), 
    .O(DayD), 
    .E(LDay), 
    .C(ED),
	 .V()
    );

endmodule
